<?php
defined('ABSPATH') or exit();

/**
 * Redforts Oscar Hotel Booking
 * Set up and initialize the plugin
 *
 * @author Redforts Software S.L.
 * @copyright Copyright (c) 2015 Redforts Software S.L.
 * @link https://redforts.com
 * @since 0.1.1
 */
class OHBE_Main
{
    /**
     * The single instance of the class.
     */
    public static $instance;

    /**
     * Define whether a search form has been display at least once.
     */
    public static $isSearchFormDisplayed;

    /**
     * Set up needed actions/filters for the plugin to initialize.
     *
     * @return void
     */
    public function __construct() {
        add_action('plugins_loaded', array($this, 'setConstants'), 1);
        add_action('plugins_loaded', array($this, 'loadLanguages'), 6);
        add_action('plugins_loaded', array($this, 'loadFiles'), 3);
        add_action('wp_enqueue_scripts', array($this, 'loadAssets'), 20);
        add_action(
            'upgrader_process_complete',
            array('OscarAPI', 'getInventoryData'),
            10,
            2
        );
        add_action('wp_head', array($this, 'setDynamicStyles'), 21);
        add_action('init', array($this, 'sessionStart'));
        if (is_admin()) {
            OHBE_Admin::getInstance();
        }
    }

    /**
     * Add hook on plugin activation.
     *
     * @return void
     */
    public static function activation() {
        self::initializeSettings();
        self::checkOhbePage();
        delete_option('ohbe_inventory');
        flush_rewrite_rules();
    }

    public static function checkOhbePage() {
        $ohbe_pages = self::getOhbePages();

        if (
            empty($ohbe_pages) ||
            (
                function_exists('pll_home_url')
                && count($ohbe_pages) !== pll_languages_list()
            )
        ) {
            self::removeOhbePages($ohbe_pages);
            self::createOhbePage();
        }
    }

    /**
     * Create OHBE page.
     *
     * @return void
     */
    public static function createOhbePage() {
        $current_user = wp_get_current_user();
        $page = array(
            'post_author' => $current_user->ID,
            'post_name' => 'ohbe',
            'post_status' => 'publish',
            'post_title'  => 'Oscar Hotel Booking Engine',
            'post_type'   => 'page',
        );

        if (
            function_exists('pll_home_url')
            && $page_id = wp_insert_post($page, true)
        ) {
            // Create pages for every language in Polylang
            $language_slugs = pll_languages_list();
            $translated_pages = array();

            foreach ($language_slugs as $slug) {
                if ($slug !== pll_default_language()) {
                    $translated_pages[$slug] = wp_insert_post($page, true);
                }
            }

            if (function_exists('pll_set_post_language')
                && function_exists('pll_save_post_translations')) {

                pll_set_post_language($page_id, pll_default_language());
                foreach ($translated_pages as $slug => $value) {
                    pll_set_post_language($value, $slug);
                }

                pll_save_post_translations(array_merge(
                    array($page_id),
                    $translated_pages
                ));
            }
        } else {
            // Create a single page without languages
            wp_insert_post($page);
        }
    }

    /**
     * Add hook on plugin deactivation.
     *
     * @return void
     */
    public static function deactivation() {
        self::removeOhbePages(self::getOhbePages());
    }

    /**
     * Create an instance of the class.
     *
     * @return object OHBE_Main instance.
     */
    public static function getInstance() {
        if (is_null(self::$instance)) {
            self::$instance = new self;
        }
        return self::$instance;
    }

    /**
     * Get inventory data.
     *
     * @return mixed Value from inventory.
     */
    public static function getInventory($field, $subfield = null) {
        $ohbe_settings = get_option('ohbe_inventory');
        if (isset($ohbe_settings[$field])) {
            $ohbe_settings_field = $ohbe_settings[$field];

            if (isset($subfield)) {
                return $ohbe_settings_field[$subfield];
            }
        }

        return isset($ohbe_settings_field) ? $ohbe_settings_field : null;
    }

    /**
     * Get OHBE pages.
     *
     * @return array OHBE pages list
     */
    public static function getOhbePages() {
        global $wpdb;
        return $wpdb->get_results(
            "
                SELECT * FROM $wpdb->posts
                WHERE post_name LIKE 'ohbe%'
                AND post_type = 'page'
                AND post_status = 'publish';
                "
        );
    }

    /**
     * Get isSearchFormDisplayed value.
     *
     * @return bool
     */
    public static function getIsSearchFormDisplayed() {
        return self::$isSearchFormDisplayed;
    }

    /**
     * Get a single field from settings.
     *
     * @return mixed Value from settings.
     */
    public static function getSettings($field) {
        $ohbe_settings = get_option('ohbe_settings');
        return isset($ohbe_settings[$field]) ? $ohbe_settings[$field] : '';
    }

    /**
     * Initialize settings data.
     *
     * @return void
     */
    private static function initializeSettings() {
        $ohbe_settings = array(
            'api_code' => '',
            'status' => "confirmed",
            'notes' => false,
            'conditions' => false,
            'only_left' => 5,
            'btn_font_color' => '#fff',
            'btn_bg_color' => '#000',
            'modify_btn_font_color' => '#fff',
            'modify_btn_bg_color' => '#fd971f',
            'promo_tag_font_color' => '#fff',
            'promo_tag_bg_color' => '#279eff',
            'best_price_tag_font_color' => '#fff',
            'best_price_tag_bg_color' => '#11a005',
            'only_left_tag_font_color' => '#fff',
            'only_left_tag_bg_color' => '#ff6767',
        );
        add_option('ohbe_settings', $ohbe_settings);
    }

    /**
     * Check if current WP version is compatible
     * with Redforts Oscar Hotel Booking API.
     *
     * @return boolean
     */
    public function isCompatibleWpVersion() {
        global $wp_version;
        return version_compare($wp_version, '4.2.2') >= 0;
    }

    /**
     * Load assets.
     *
     * @return void
     */
    public function loadAssets() {
        if (!is_admin()) {
            // Javascript
            wp_enqueue_script(
                'ohbe_moment',
                '//cdnjs.cloudflare.com/ajax/libs/moment.js/2.22.2/'
                . 'moment-with-locales.min.js',
                array('jquery')
            );
            wp_enqueue_script('jquery-ui-core', array('jquery'));
            wp_enqueue_script('jquery-ui-datepicker', array('jquery'));
            wp_enqueue_script(
                'ohbe_common',
                OHBE_URL . 'assets/js/ohbe-common.js',
                array('jquery', 'jquery-ui-datepicker'),
                OHBE_VERSION,
                1
            );
            wp_register_script(
                'ohbe_availability',
                OHBE_URL . 'assets/js/ohbe-availability.js',
                array('jquery'),
                OHBE_VERSION,
                1
            );
            wp_register_script(
                'ohbe_carousel',
                OHBE_URL . 'assets/js/ohbe-carousel.js',
                array('jquery'),
                OHBE_VERSION,
                1
            );
            wp_register_script(
                'ohbe_reservation',
                OHBE_URL . 'assets/js/ohbe-reservation.js',
                array('jquery'),
                OHBE_VERSION,
                1
            );
            wp_register_script(
                'ohbe_reservation_data',
                OHBE_URL . 'assets/js/ohbe-reservation-data.js',
                array('jquery'),
                OHBE_VERSION,
                1
            );
            wp_localize_script(
                'ohbe_common',
                'dp_args',
                OHBE_Tools::localizeDatepicker()
            );
            wp_localize_script(
                'ohbe_common',
                'search_filters',
                OHBE_Tools::getSearchFilters('d-m-Y')
            );
            wp_localize_script(
                'ohbe_common',
                'translation',
                array('nights' => __('nights', 'ohbe'))
            );
            wp_localize_script(
                'ohbe_reservation',
                'ohbe_field_errors',
                array(
                    'occupation_exceed' =>
                    __('The number of adults and children is not allowed for the '
                       . 'selected rate and accommodation.', 'ohbe'),
                    'empty_fields' => __('Please fill out the required fields.', 'ohbe'),
                )
            );
            // CSS
            wp_enqueue_style(
                'jquery-ui-css',
                '//ajax.googleapis.com/ajax/libs/jqueryui/1.8.2/themes/'
                . 'smoothness/jquery-ui.css'
            );
            wp_enqueue_style(
                'ohbe_style', OHBE_URL .
                'assets/css/style.css',
                '',
                OHBE_VERSION
            );
        }
    }

    /**
     * Load files.
     *
     * @return void
     */
    public function loadFiles() {
        require_once(OHBE_PATH . 'includes/api/OscarAPI.php');
        require_once(OHBE_PATH . 'includes/class/ohbe-availability.php');
        require_once(OHBE_PATH . 'includes/class/ohbe-reservation.php');
        require_once(OHBE_PATH . 'includes/class/ohbe-review.php');
        require_once(OHBE_PATH . 'includes/class/ohbe-tools.php');
        require_once(OHBE_PATH . 'includes/functions.php');
        require_once(OHBE_PATH . 'includes/templates/OHBE-virtual-page-creator.php');
        require_once(OHBE_PATH . 'includes/templates/shortcode/ohbe-search.php');
        require_once(OHBE_PATH . 'includes/templates/shortcode/ohbe-show.php');
        require_once(OHBE_PATH . 'includes/admin/widgets/ohbe-search-widget.php');
        require_once(OHBE_PATH . 'includes/admin/widgets/ohbe-show-widget.php');
    }

    /**
     * Load language files.
     *
     * @return void
     */
    public function loadLanguages() {
        load_plugin_textdomain(
            'ohbe',
            false,
            plugin_basename(OHBE_PATH . 'languages')
        );
        load_plugin_textdomain(
            'ohbe-countries',
            false,
            plugin_basename(OHBE_PATH . 'languages/countries')
        );
    }

    /**
     * Remove OHBE pages.
     *
     * @param array $ohbe_pages OHBE pages list.
     * @return void
     */
    public static function removeOhbePages($ohbe_pages) {
        if (!empty($ohbe_pages)) {
            if (function_exists('pll_get_post_translations')) {
                // Delete pages for every language in Polylang
                foreach ($ohbe_pages as $ohbe_page) {
                    wp_delete_post($ohbe_page->ID, true);
                }
            } else {
                // Delete a single page without languages
                wp_delete_post($ohbe_pages[0]->ID, true);
            }
        }
    }

    /**
     * Define constants for the plugin.
     *
     * @return void
     */
    public function setConstants() {
        // Set constant path to availability page
        define('OHBE_AVAILABILITY_PAGE', 'availability');
        // Set constant path to reservation complete page
        define('OHBE_RESERVATION_COMPLETE_PAGE', 'reservation-complete');
        // Set constant path to reservation page
        define('OHBE_RESERVATION_PAGE', 'reservation');
        // Set constant path to review page
        define('OHBE_REVIEW_PAGE', 'review');
    }

    /**
     * Add dynamic styles to <head>.
     *
     * @return void
     */
    public static function setDynamicStyles() {
        // Btn
        $html = sprintf('<style>
            button.btn-ohbe,
            .ohbe-page #next-btn,
            .ohbe-page .btn-select-rate,
            .ohbe-page .rate-list .rate:hover,
            .ohbe-page .rate-list .rate.selected,
            .ohbe-page .btn-select-quantity,
            .ohbe-page .quantity-list .quantity:hover,
            .ohbe-page .quantity-list .quantity.selected,
            .ohbe-page .ohbe-accordion.ui-accordion .ui-accordion-header:after,
            .ohbe-page .item-element .quantity-total-block button {
                background-color: %1$s;
                color: %2$s;
            }
            .ohbe-page .btn-select-rate:hover,
            .ohbe-page .btn-select-rate:focus,
            .ohbe-page .btn-select-quantity:hover,
            .ohbe-page .btn-select-quantity:focus {
                background-color: %1$s;
            }
            .ohbe-page .btn-select-rate .dashicons,
            .ohbe-page .btn-select-quantity .dashicons {
                color: %2$s;
            }
            .ohbe-page .acco-title .dashicons,
            .ohbe-page .info .conditions .dashicons {
                color: %1$s;
            }
            ',
            OHBE_Main::getSettings('btn_bg_color'),
            OHBE_Main::getSettings('btn_font_color')
        );
        $html .= sprintf('
            .accommodation-element .acco-info .rate-block .quantity-total-block
            .accos-quantity .value-wrapper::after {
                color: %s;
            }',
            OHBE_Main::getSettings('btn_font_color')
        );
        // Modify Btn
        $html .= sprintf('.ohbe-page .btn-modify {
                background-color: %1$s;
                color: %2$s;
            }
            .reservation-data-panel .data-group .btn-modify {
                color: %1$s;
            }
            ',
            OHBE_Main::getSettings('modify_btn_bg_color'),
            OHBE_Main::getSettings('modify_btn_font_color')
        );
        // Promo tag
        $html .= sprintf('.ohbe-page .total-promo .tag,
            .ohbe-page .promo .tag {
                background-color: %1$s;
                color: %2$s;
            }
            .ohbe-page .total-promo .tag:before,
            .ohbe-page .tag:before {
                background-color: #fff;
            }
            .ohbe-page .acco-info .rate-block .total-promo .tag:after,
            .ohbe-page .acco-info .rate-block .promo .tag:after,
            .ohbe-page .alert-promo-amount-discount .tag:after {
                border-left-color: %1$s;
            }
            .ohbe-page .ribbon {
                color: %2$s;
                background-color: %1$s;
                -webkit-box-shadow: 0 1px 4px %1$s;
                -moz-box-shadow: 0 1px 4px %1$s;
                box-shadow: 0 1px 4px %1$s;
            }
            .ohbe-page .ribbon:after {
                border-color: %1$s transparent transparent %1$s;
            }
            ',
            OHBE_Main::getSettings('promo_tag_bg_color'),
            OHBE_Main::getSettings('promo_tag_font_color')
        );
        // Best price tag
        $html .= sprintf('.ohbe-page .tag.best-price {
                background-color: %1$s;
                color: %2$s;
            }
            .ohbe-page .acco-info .rate-block .tag.best-price:after {
                border-left-color: %1$s;
            }',
            OHBE_Main::getSettings('best_price_tag_bg_color'),
            OHBE_Main::getSettings('best_price_tag_font_color')
        );
        // Only left tag
        $html .= sprintf('.ohbe-page .only-left {
                background-color: %1$s;
                color: %2$s;
            }
            </style>',
            OHBE_Main::getSettings('only_left_tag_bg_color'),
            OHBE_Main::getSettings('only_left_tag_font_color')
        );
        echo $html;
    }

    /**
     * Start session.
     *
     * @return void
     */
    public function sessionStart() {
        if (!session_id()) {
            session_start();
        }

        if (!isset($_SESSION['ohbe'])) {
            $_SESSION['ohbe'] = array();
        }
    }

    /**
     * Set isSearchFormDisplayed value.
     *
     * @return void
     */
    public static function setIsSearchFormDisplayed($value) {
        self::$isSearchFormDisplayed = $value;
    }

    /**
     * Set settings data.
     *
     * @return void
     */
    public static function setSettings($field, $value) {
        $ohbe_settings = get_option('ohbe_settings');
        $ohbe_settings[$field] = $value;
        update_option('ohbe_settings', $ohbe_settings);
    }

    /**
     * Add hook on plugin uninstall.
     *
     * @return void
     */
    public static function uninstall() {
        delete_option('ohbe_settings');
        delete_option('ohbe_inventory');
        flush_rewrite_rules();
    }
}
