<?php
defined('ABSPATH') or exit();

/**
 * Redforts Oscar Hotel Booking
 * API class
 *
 * @author Redforts Software S.L.
 * @copyright Copyright (c) 2015 Redforts Software S.L.
 * @link https://redforts.com
 * @since 0.1.1
 */

class OscarAPI
{
    const API_VERSION = 5;

    /**
     * Get the available accommodations in Oscar.
     *
     * @param string $arrival Arrival date. Format: Y-m-d.
     * @param string $departure Departure date. Format: Y-m-d.
     * @param string $promo_code Promotion code.
     * @return array Oscar availability call response.
     */
    public static function getAvailability($arrival, $departure, $promo_code="") {
        if (OHBE_Main::getSettings('api_code')) {
            $body = array(
                "api_version" => self::API_VERSION,
                "arrival" => $arrival,
                "departure" => $departure,
                "lang" => self::getLanguage(),
                "promo" => $promo_code,
                "system_info" => array(
                    "ohbe" => OHBE_VERSION,
                    "os" => php_uname('s') . ' ' . php_uname('r'),
                    "php" => phpversion(),
                    "wordpress" => get_bloginfo('version'
                ),
            ));

            return OscarAPI::send('availability', $body);
        }
    }

    /**
     * Get the calendar dates availability in Oscar.
     *
     * @return array Oscar calendar data call response.
     */
    public static function getCalendarData() {
        if (OHBE_Main::getSettings('api_code')) {
            $body = array(
                "api_version" => self::API_VERSION,
                );

            return OscarAPI::send('calendar_data', $body);
        }
    }

    /**
     * Get the inventory in Oscar.
     *
     * @return array Oscar inventory call response.
     */
    public static function getInventory() {
        if (OHBE_Main::getSettings('api_code')) {
            $body = array(
                "api_version" => self::API_VERSION,
            );

            return OscarAPI::send('inventory', $body);
        }
    }

    /**
     * Get the available item list in Oscar.
     *
     * @return array Oscar item list data call response.
     */
    public static function getItemList() {
        if (OHBE_Main::getSettings('api_code')) {
            extract(OHBE_Reservation::getReservationParam());
            $body = array(
                "api_version" => self::API_VERSION,
                "arrival" => $arrival->format('Y-m-d'),
                "name" => $name,
                "reserv_id" => intval($reserv_id),
            );

            return OscarAPI::send('item_list', $body);
        }
    }

    /**
     * Update inventory data from Oscar.
     *
     * @return void
     */
    public static function getInventoryData() {
        $inventory = OscarAPI::getInventory();

        if (!isset($inventory) || isset($inventory['errors'])) {
            OHBE_Tools::setFlashMessage($inventory['errors'][0]['text'], 'error');
            return;
        }
        update_option('ohbe_inventory', $inventory);
    }

    /**
     * Set a language for API calls to Oscar based on WP locale.
     *
     * @return string|null Locale slug. Example: 'en'
     */
    public static function getLanguage() {
        $inv_langs = OHBE_Main::getInventory('languages');

        if ($inv_langs) {
            foreach ($inv_langs as $code => $data) {
                // Polylang
                if(
                    function_exists('pll_current_language')
                    && $code == pll_current_language()
                ) {
                    return pll_current_language();
                }
                // Locale
                if (
                    $code == substr(get_locale(), 0, 2)
                    || $code == get_locale()
                ) {
                    return $code;
                }
            }
        }

        return OHBE_Main::getInventory('default_lang');
    }

    /**
     * Get a reservation in Oscar.
     *
     * @return array Oscar reservation data call response.
     */
    public static function getReservationData() {
        if (OHBE_Main::getSettings('api_code')) {
            extract(OHBE_Reservation::getReservationParam());
            $body = array(
                "api_version" => self::API_VERSION,
                "arrival" => $arrival->format('Y-m-d'),
                "name" => $name,
                "lang" => self::getLanguage(),
                "reserv_id" => intval($reserv_id),
            );

            return OscarAPI::send('reservation_data', $body);
        }
    }

    /**
     * Get the full URL for the API call.
     *
     * @param string $call API call.
     * @return string|null Locale slug. Example: 'en'
     */
    public static function getUrl($call) {
        return OHBE_BASE_URI . OHBE_Main::getSettings('api_code') . '/' . $call;
    }

    /**
     * Update the item list for reservation in Oscar.
     *
     * @param array $item Item to update.
     * @return array Oscar reservation call response
     */
    public static function setItemsUpdate($item) {
        if (OHBE_Main::getSettings('api_code')) {
            extract(OHBE_Reservation::getReservationParam());
            $body = array(
                'api_version' => self::API_VERSION,
                'arrival' => $arrival->format('Y-m-d'),
                "name" => $name,
                'items' => array($item),
                'lang' => self::getLanguage(),
                'reserv_id' => intval($reserv_id),
            );

            return OscarAPI::send('items_update', $body);
        }
    }

    public static function send($call, $body) {
        return json_decode(
            wp_remote_retrieve_body(
                wp_remote_post(
                    OscarAPI::getUrl($call),
                    array(
                        'headers' => array(
                            'Content-Type' => 'application/json; charset=utf-8'
                        ),
                        'body' => json_encode($body),
                    )
                )
            ),
            true
        );
    }

    /**
     * Make a reservation in Oscar.
     *
     * @param string $arrival Arrival date. Format: Y-m-d.
     * @param string $departure Departure date. Format: Y-m-d.
     * @param string $status Status of the reservation.
     * @param array $accos Accommodations.
     * @param array $reservation_params Reservation params.
     * @param string $promo_code Promotion code.
     * @return array Oscar reservation call response
     */
    public static function setReservation(
        $arrival,
        $departure,
        $status,
        $accos,
        $reservation_params,
        $promo_code=''
    ) {
        if (OHBE_Main::getSettings('api_code')) {
            $body = array(
                'api_version' => self::API_VERSION,
                'arrival' => $arrival,
                'departure' => $departure,
                'lang' => self::getLanguage(),
                'promo' => $promo_code,
                'status' => $status,
                'accos' => $accos,
            );
            foreach ($reservation_params as $key => $value) {
                $body[substr($key, 5)] = $value;
            }

            return OscarAPI::send('reservation', $body);
        }
    }
}
