<?php
defined('ABSPATH') or exit();

/**
 * Redforts Oscar Hotel Booking
 * Availability functions
 *
 * @author Redforts Software S.L.
 * @copyright Copyright (c) 2015 Redforts Software S.L.
 * @link https://redforts.com
 * @since 0.1.1
 */
class OHBE_Availability
{
    /**
     * Handle accommodation availability errors if set.
     *
     * @return object|null Error object or null.
     */
    public static function accommodationAvailabilityErrors($type, $filters=null) {
        $errors = new WP_Error();

        switch($type) {
            case 'no_filters':
                $errors->add(
                    'no_filters',
                    __('Please introduce the arrival and departure dates', 'ohbe')
                );
                break;
            case 'invalid_param':
                $errors->add(
                    'invalid_param',
                    __('Invalid search parameter', 'ohbe')
                );
            case 'position':
                if (
                    $filters['arrival']->format('Y-m-d')
                    >= $filters['departure']->format('Y-m-d')
                ) {
                    $errors->add(
                        'ohbe_wrong_date_position',
                        __('Departure not after arrival.', 'ohbe')
                    );
                }
            default:
                return null;
        }

        return $errors;
    }

    /**
     * Create the accommodation array structure.
     *
     * @param array $redforts_data Oscar availability call response.
     * @param array $all_filters Filters set.
     * @return array Accommodations structure.
     */
    public static function accommodationsHandle($redforts_data, $all_filters) {
        $accos = array();
        if (isset($redforts_data['options'])) {
            foreach ($redforts_data['options'] as $option) {
                $acco_arrival = DateTime::createFromFormat(
                    '!Y-m-d',
                    $option['arrival']
                );
                $acco_departure = DateTime::createFromFormat(
                    '!Y-m-d',
                    $option['departure']
                );
                $range_type = 'selected';

                if (
                    $acco_arrival > $all_filters['arrival']
                    || $acco_departure < $all_filters['departure']
                ) {
                    $range_type = 'limited';
                }
                if (
                    $acco_arrival < $all_filters['arrival']
                    || $acco_departure > $all_filters['departure']
                ) {
                    $range_type = 'suggested';
                }
                if (
                    (isset($_GET['acco']) && $option['acco_id'] == $_GET['acco'])
                    || !isset($_GET['acco'])
                ) {
                    $promo_code = isset($all_filters['pr'])
                        ? $all_filters['pr'] : null;
                    self::filterAccommodations(
                        $option,
                        $accos,
                        $range_type,
                        $promo_code
                    );
                }
            }
        }

        return $accos;
    }

    /**
     * Add clearfix to accommodation card for grid.
     *
     * @param int $count
     * @return void
     */
    public static function printClearfix($count) {
        if ($count % 2 == 0) {
            echo '<li class="clearfix visible-sm-block hidden-md hidden-lg"></li>';
        }
        if ($count % 3 == 0) {
            echo '<li class="clearfix hidden-sm visible-md-block visible-lg-block">'
                . '</li>';
        }
    }

    /**
     * Create period structure for an accommodation.
     *
     * @param array $option Accommodation available in Oscar.
     * @param string $range_type Range of days available
     *      ("selected", "limited", "suggested").
     * @param array $accos Accommodations structure.
     * @param string $promo_code Promo code if set.
     * @param bool $new_acco Tell if the accommodation is set or not.
     * @return void
     */
    public static function createPeriods(
        $option,
        $range_type,
        &$accos,
        $promo_code,
        &$new_acco=false
    ) {
        $count = 0;
        $acco_id = $option['acco_id'];
        $price = (string)$option['price'];
        $range = $option['arrival'] . '_' . $option['departure'];
        $p = 'periods';
        if ($new_acco) {
            foreach ($option['periods'] as $period) {
                $period['arrival'] = $option['arrival'];
                $period['departure'] = $option['departure'];
                $period['adults'] = $option['adults'];
                $period['children'] = $option['children'];
                $period['excl_taxes'] = $option['excl_taxes'];
                $period['night_string'] = sprintf(
                    _n('%s night', '%s nights', $period['days'], 'ohbe'),
                    $period['days']
                );
                if (isset($option['orig_price']) && !$new_acco['promo_code']) {
                    $new_acco['promo_code'] = $promo_code;
                }
                if (isset($option['promo_amount'])) {
                    // Set promo_amount to period for Fixed-amount Promo Code
                    $period['promo_amount'] = $option['promo_amount'];
                }
                if ($count == 0) {
                    $period['sorted'] = 0;
                    $new_acco[$p][$price][][] = $period;
                }
                else {
                    $sorted = count($new_acco[$p][$price]) - 1;
                    $period['sorted'] = $sorted;
                    $new_acco[$p][$price][$sorted][] = $period;
                }
                $count++;
            }
            if (isset($option['promo_amount'])) {
                // Set promo_code to acco object for Fixed-amount Promo Code
                $new_acco['promo_code'] = $promo_code;
            }
            $accos[$range_type][$range][$acco_id] = $new_acco;
        }
        else {
            $acco = $accos[$range_type][$range][$acco_id];
            foreach ($option['periods'] as $period) {
                $period['arrival'] = $option['arrival'];
                $period['departure'] = $option['departure'];
                $period['adults'] = $option['adults'];
                $period['children'] = $option['children'];
                $period['excl_taxes'] = $option['excl_taxes'];
                $period['night_string'] = sprintf(
                    _n('%s night', '%s nights', $period['days'], 'ohbe'),
                    $period['days']
                );
                if (isset($option['promo_amount'])) {
                    // Set promo_amount to period for Fixed-amount Promo Code
                    $period['promo_amount'] = $option['promo_amount'];
                }
                if (
                    isset($option['orig_price'])
                    && !isset($acco['promo_code'])
                ) {
                    $acco['promo_code'] = $promo_code;
                }
                if ($count == 0) {
                    $sorted = isset($acco[$p][$price])
                        ? count($acco[$p][$price])
                        : 0;
                    $period['sorted'] = $sorted;
                    $acco[$p][$price][][] = $period;
                }
                else {
                    $sorted = count($acco[$p][$price]) - 1;
                    $period['sorted'] = $sorted;
                    $acco[$p][$price][$sorted][] = $period;
                }
                $count++;
            }
            if ($price < $acco['price']) {
                // Set the accommodation lowest price/orig_price by default
                // If option has orig_price add it to the accommodation, otherwise
                // is set as null. Doing so, price and orig_price don't get mixed
                // from different options.
                $acco['price'] = $price;
                $acco['orig_price'] =
                    isset($option['orig_price']) ? $option['orig_price'] : null;

                $acco['promo_amount'] =
                    isset($option['promo_amount'])
                        ? $option['promo_amount']
                        : null;

            }
            $accos[$range_type][$range][$acco_id] = $acco;
        }
    }

    /**
     * @param array $option Accommodation available in Oscar.
     * @param string $range_type Range of days available
     *      ("selected", "limited", "suggested").
     * @param array $accos Accommodations structure.
     * @param string $promo_code Promo code if set.
     * @return void
     */
    public static function filterAccommodations(
        $option,
        &$accos,
        $range_type,
        $promo_code=null
    ) {
        $range = $option['arrival'] . '_' . $option['departure'];

        if (!isset($accos[$range_type][$range][$option['acco_id']])) {
            $nights_number = OHBE_Tools::getNightsInterval($option);
            $new_acco = new ArrayObject($option);
            $new_acco['date_range'] = $range_type;
            $new_acco['nights'] = $nights_number;
            $new_acco['night_string'] = sprintf(
                _n('%s night', '%s nights', $nights_number, 'ohbe'),
                $nights_number
            );
            $new_acco['orig_price'] = isset($option['orig_price'])
                    ? $option['orig_price']
                    : null;
            $new_acco['promo_amount'] = isset($option['promo_amount'])
                    ? $option['promo_amount']
                    : null;
            $new_acco['periods'] = array();
            $new_acco['promo_code'] = null;
            if (isset($option['name'])) {
                $new_acco['name'] = $option['name'];
            }
            self::createPeriods(
                $option,
                $range_type,
                $accos,
                $promo_code,
                $new_acco
            );
        }
        else{
            self::createPeriods($option, $range_type, $accos, $promo_code);
        }
    }

    /**
     * Get accommodations panel printed in the page.
     *
     * @param array $redforts_data Oscar availability call response.
     * @param array $all_filters Filters set.
     * @return array|null
     */
    public static function getAccommodationsStructure($redforts_data, $all_filters) {
        if (ohbeAvailabilityFormHandle()) {
            return null;
        }
        $updated_oscar_data = OHBE_Tools::checkVersion(
            $redforts_data['inventory_version']
        );
        if ($updated_oscar_data) {
            OscarAPI::getInventoryData();
        }
        return OHBE_Availability::accommodationsHandle(
            $redforts_data,
            $all_filters
        );
    }

    /**
     * Print accommodation card's attributes.
     *
     * @param  array $acco Single accommodation structure.
     * @return void
     */
    public static function printAccommodationCardData($acco) {
        $inv_langs = OHBE_Main::getInventory('languages');
        $lang = OscarAPI::getLanguage();
        $name = isset($inv_langs[$lang]['accos'][$acco['acco_id']]['name'])
            ? $inv_langs[$lang]['accos'][$acco['acco_id']]['name']
            : OHBE_Main::getInventory('accos')[$acco['acco_id']]['name'];

        printf(
            'data-name="%s" data-range=%s data-id=%s data-amount=%s '
            . 'data-arrival=%s data-departure=%s data-price=%s data-orig-price="%s" '
            . 'data-promo-amount="%s" data-taxes="%s" data-nights=%s '
            . 'data-night-string="%s" data-rate=%s data-sorted=%s %s %s',
            $name,
            $acco['date_range'],
            $acco['acco_id'],
            $acco['amount'],
            $acco['arrival'],
            $acco['departure'],
            $acco['price'],
            isset($acco['orig_price']) ? $acco['orig_price'] : '',
            isset($acco['promo_amount']) ? $acco['promo_amount'] : '',
            $acco['excl_taxes'],
            $acco['nights'],
            $acco['night_string'],
            OHBE_Availability::renderDefaultRatePartIds($acco['periods']),
            "0",
            OHBE_Availability::renderDefaultAccommodationRateAdultsAndChildren(
                $acco['periods']
            ),
            isset($acco['promo_code'])
                ? sprintf('data-promo-code=%s', $acco['promo_code'])
                : ''
        );
    }

    /**
     * Print available accommodations.
     *
     * @param array $redforts_data Oscar availability call response.
     * @param array $accommodations Accommodations structure.
     * @param array $all_filters Filters set.
     * @return void
     */
    public static function printAccommodationLists(
        $redforts_data,
        $accommodations,
        $all_filters
    ) {
        if (
            !isset($redforts_data['errors'])
            && (empty($redforts_data['options']) || empty($accommodations))
        ) {
            // There are no accommodations available.
            OHBE_Tools::setFlashMessage(
                __(
                    'There are no accommodations available for the selected dates.',
                    'ohbe'
                )
                . ' '
                . __('Please try another period.', 'ohbe'),
                'danger'
            );
            OHBE_Tools::printFlashMessage();
        }
        else if (
            $accommodations
            && count($accommodations) > 0
            && !isset($accommodations['selected'])
        ) {
            // There are no accommodations available in the selected dates,
            // but there are available in another dates.
            printf(
                '<div class=ohbe-noaccos><span>%s</span><span>%s</span></div>',
                __('There are no accommodations available for the selected dates.',
                    'ohbe'),
                __('Check out the options below or try another period.', 'ohbe')
            );
        }
        if ($accommodations) {
            // There are accommodations available in the selected dates.
            // Javascript
            wp_enqueue_script('jquery-ui-accordion');
            wp_enqueue_script('ohbe_moment');
            wp_enqueue_script('ohbe_availability');
            wp_enqueue_script('ohbe_carousel');
            wp_localize_script(
                'ohbe_availability',
                'accommodations',
                $accommodations
            );
            // These values are used in accommodation_card.php
            $promo_code = isset($all_filters['pr']) ? $all_filters['pr'] : null;

            if (isset($accommodations['selected'])) {
                usort(
                    $accommodations['selected']
                    [current(array_keys($accommodations['selected']))],
                    array('OHBE_Tools','sortByPrice')
                );
                printf(
                    '<div class=availability-header>'
                    . '<h3 class=acco-list-title>%s</h3>'
                    . '</div>',
                    __('Select accommodations', 'ohbe')
                );
                foreach ($accommodations['selected'] as $acco_date) {
                    print('<ul class="accommodation-list">');
                    $count = 1;
                    foreach ($acco_date as $acco) {
                        ksort($acco['periods']);
                        include(
                            OHBE_PATH . 'includes/templates/views/accommodation_card.php'
                        );
                        $count++;
                    }
                    print('</ul>');
                }
            }
            if (isset($accommodations['limited'])) {
                foreach ($accommodations['limited'] as $key => $value) {
                    usort(
                        $accommodations['limited'][$key],
                        array('OHBE_Tools','sortByPrice')
                    );
                    $dates = explode('_', $key);
                    if (isset($dates[0]) && isset($dates[1])) {
                        $arrival = DateTime::createFromFormat('Y-m-d', $dates[0]);
                        $departure = DateTime::createFromFormat('Y-m-d', $dates[1]);
                        printf(
                            '<div class="alert alert-success">%s: %s - %s</div>'
                            . '<ul class="accommodation-list">',
                            __('Limited availability', 'ohbe'),
                            $arrival->format('d/m/Y'),
                            $departure->format('d/m/Y')
                        );
                        $count = 1;
                        foreach ($value as $acco) {
                            ksort($acco['periods']);
                            include(
                                OHBE_PATH . 'includes/templates/'
                                . 'views/accommodation_card.php'
                            );
                            $count++;
                        }
                        print('</ul>');
                    }
                }
            }
            if (isset($accommodations['suggested'])) {
                foreach ($accommodations['suggested'] as $key => $value) {
                    usort(
                        $accommodations['suggested'][$key],
                        array('OHBE_Tools','sortByPrice')
                    );
                    $dates = explode('_', $key);
                    if (isset($dates[0]) && isset($dates[1])) {
                        $arrival = DateTime::createFromFormat('Y-m-d', $dates[0]);
                        $departure = DateTime::createFromFormat('Y-m-d', $dates[1]);
                        printf(
                            '<div class=ohbe-accordion>'
                            . '<div class="alert alert-availability">'
                            . '<span class=range>%s</span> '
                            . '<span class=dates>%s - %s</span></div>'
                            . '<div>' // accordion content will be set to display block when displaying
                            . '<ul class="accommodation-list">',
                            __('Offers for other dates', 'ohbe'),
                            $arrival->format('d/m/Y'),
                            $departure->format('d/m/Y')
                        );
                        $count = 1;
                        foreach ($value as $acco) {
                            ksort($acco['periods']);
                            include(
                                OHBE_PATH . 'includes/templates/views/'
                                . 'accommodation_card.php'
                            );
                            $count++;
                        }
                        print('</ul></div></div>');
                    }
                }
            }
        }
    }

    /**
     * Handle error alert(s) when exist.
     *
     * @param array $redforts_data Oscar availability call response.
     * @return void
     */
    public static function handleErrors($redforts_data) {
        if (isset($redforts_data['errors'])) {
            foreach ($redforts_data['errors'] as $error) {
                OHBE_Tools::setFlashMessage($error['text'], 'danger');
                OHBE_Tools::printFlashMessage();
            }
        }
    }

    /**
     * Print alert when there are no accommodations available.
     *
     * @param array $redforts_data Oscar availability call response.
     * @param array $accos Accommodations structure.
     * @return void
     */
    public static function printNoAccommodationsAvailable($redforts_data, $accos) {
        if (
            !isset($redforts_data['errors'])
            && (empty($redforts_data['options']) || empty($accos))
        ) {
            OHBE_Tools::setFlashMessage(
                __(
                    'There are no accommodations available for the selected dates.',
                    'ohbe'
                )
                . ' '
                . __('Please try another period.', 'ohbe'),
                'danger'
            );
            OHBE_Tools::printFlashMessage();
        }
    }

    public static function printPromoMessage($redforts_data) {
        if (isset($redforts_data['promo_total'])) {
            $currency = OHBE_Main::getInventory('currency');
            echo '<div class="alert-promo-amount-discount promo">'
                . '<div class="tag">' . $_GET['pr'] . '</div>'
                . '<div class="value">';
            printf(__(
                    'Select these options and get a discount of up to %d %s.',
                    'ohbe'
                ),
                $redforts_data['promo_total'],
                $currency['symbol']
            );
            echo  '</div></div>';
        }
    }

    /**
     * Print available accommodations previously selected.
     *
     * @param array $redforts_data Oscar availability call response.
     * @return void
     */
    public static function printSelectedAccommodations(
        $redforts_data
    ) {
        $total_price = $promo_amount_total = 0;

        echo '<h3 class=ohbe-selected-list-title>'
            . __('Selected accommodations', 'ohbe')
            . '</h3>'
            . '<form id=ohbe-booking-summary-form method=POST action='
            . OHBE_Tools::renderFormAction(OHBE_RESERVATION_PAGE, true) .'>'
            . '<ul class=accommodation-booked-list>';

        if (isset($_SESSION['ohbe']['selected_accos'])) {
            $isRemovable = true;
            foreach ($_SESSION['ohbe']['selected_accos'] as $acco) {
                // Comparisson between selected accos and accos loaded by Oscar is
                // needed in order to known if the accommodation is still available.
                foreach ($redforts_data['options'] as $option) {
                    if (
                        $acco['acco_id'] == $option['acco_id']
                        && $acco['adults'] == $option['adults']
                        && $acco['arrival'] == htmlspecialchars($option['arrival'])
                        && $acco['children'] == $option['children']
                        && $acco['departure'] == $option['departure']
                        && $acco['excl_taxes'] == $option['excl_taxes']
                        && $acco['price'] == $option['price']
                        && $acco['quantity'] <= $option['amount']
                    ) {
                        $rate_ids = '';
                        for ($i=0; $i<count($option['periods']); $i++) {
                            if ($i > 0) {
                                $rate_ids .= ',';
                            }
                            $rate_ids .= $option['periods'][$i]['rate_id'];
                        }
                        if ($acco['rate'] == $rate_ids) {
                            $total_price += $acco['quantity'] * $acco['price'];
                            // Sum all accommodations promo_amount
                            if ($acco['promo_amount']) {
                                $promo_amount_total += floatval(
                                    $acco['promo_amount']
                                );
                            }
                            include(
                                OHBE_PATH . 'includes/templates/views/'
                                . 'accommodation_reservation_card.php'
                            );
                        }
                    }
                }
            }
        }

        echo '</ul>';

        if (isset($redforts_data['promo_total']) && $promo_amount_total > 0) {
            // Set orig_price and new total_price
            $orig_price = $total_price;
            $total_price -= $promo_amount_total > $redforts_data['promo_total']
                ? $redforts_data['promo_total']
                : $promo_amount_total;
            if ($total_price < 0) {
                $total_price = 0;
            }
        }

        OHBE_Tools::printBookedListBottom(
            $total_price,
            'next-btn',
            'ohbe_go_reservation_page',
            __('Next', 'ohbe')
            . '<span class="dashicons dashicons-arrow-right-alt"></span>',
            true,
            isset($orig_price) ? $orig_price : null
        );

        if (isset($redforts_data['promo_total'])) {
            printf(
                '<input type=hidden id=ohbe-promo-total name=promo_total value=%s>',
                $redforts_data['promo_total']
            );
        }
        echo '<input type=hidden id=ohbe-accommodations-input name=accommodations>'
            . '</form>';
    }

    /**
     * Render accommodation card's accos-total block.
     *
     * @param  array $acco Single accommodation structure.
     * @return string Accos-total block's html structure.
     *
     */
    public static function renderAccosTotalBlock($acco) {
        $quantity_list = '';
        $default_rate_has_promo = false;
        $s_acco = OHBE_Tools::getSelectedAcco($acco);

        if ($s_acco || ($s_acco && $s_acco['promo_amount'])) {
            // An accommodation with fixed promo code is in cart or
            // an accommodation without promo code is in cart
            $first_line_price = $s_acco['price'] * $s_acco['quantity'];
        }
        if ($s_acco && $s_acco['orig_price']) {
            // An accommodation with promo code is in cart
            $first_line_price = $s_acco['orig_price'] * $s_acco['quantity'];
            $second_line_price = $s_acco['price'] * $s_acco['quantity'];
            $default_rate_has_promo = true;
        }
        else if ($acco['orig_price']) {
            // No accommodation is in cart and default acco has promo code
            $first_line_price = $acco['orig_price'];
            $second_line_price = $acco['price'];
            $default_rate_has_promo = true;
        }
        else {
            // No accommodation is in cart and default acco has no promo code
            $first_line_price = $acco['price'];
        }

        if ($acco['amount'] > 1) {
            // Show / hide quantity list depending on accommodations amount
            for ($i=1; $i<=$acco['amount']; $i++) {
                $acco_string = sprintf(
                    _n('%s accommodation', '%s accommodations', $i, 'ohbe'), $i
                );
                $quantity_list .= sprintf(
                    '<li class="quantity%s" data-quantity="%s">%s',
                    $s_acco && $s_acco['quantity'] == $i ? ' selected' : '',
                    $i,
                    $acco_string
                );
            }
        }
        $html = sprintf(
            '<div class=quantity-total-block>'
            . '<button class="btn-select-quantity %s"%s>%s',
            $quantity_list ? 'has-list' : 'has-no-list',
            $quantity_list ? '' : 'data-quantity=1',
            __('Reserve now', 'ohbe')
        );
        if ($quantity_list)
            $html .= '<span class="dashicons dashicons-arrow-down"></span>';
        $html .= sprintf(
            '</button><ul class=quantity-list>%s</ul><div class=total%s>'
            . '<div class=ohbe-title>%s:</div><div class="value%s">%s</div>',
            $quantity_list,
            is_null($s_acco) ? ' style="display:none;"' : '',
            $acco['excl_taxes']
                ? sprintf(
                esc_html(__('Total excluding %1$s%% taxes', 'ohbe')),
                $acco['excl_taxes']
            )
                : __('Total including taxes', 'ohbe'),
            $default_rate_has_promo ? ' orig' : '',
            OHBE_Tools::formatPrice($first_line_price)
        );
        $html .= sprintf(
            '<div class="total-promo"%s>'
            . '<div class="ohbe-title tag">%s</div>'
            . '<div class="value">%s</div>'
            . '</div>',
            $s_acco && isset($acco['orig_price'])
                ? ''
                : ' style="display:none;"',
            $acco['promo_code'],
            isset($second_line_price)
                ? OHBE_Tools::formatPrice($second_line_price)
                : ''
        );

        return $html;
    }

    /**
     * Render accommodation card's combined rate info block.
     *
     * @param  array $acco Single accommodation structure.
     * @param  bool $is_selected Tell if the accommodation has been selected
     * previously.
     * @param array $rates Translated rate names from Inventory.
     * @param string $first_price_key First key of periods structure.
     * @param string $total_rate_orig_price Total rate original price.
     * @param bool $has_default_rate_percentage_promo Tell if the rate has
     * percentage-based promo or not.
     * @param bool $has_default_rate_fixed_promo Tell if the rate has fixed-amount
     * @return string Accos-total block's html structure.
     *
     */
    public static function renderCombinedRateInfo(
        $acco,
        $is_selected,
        $rates,
        $first_price_key,
        &$total_rate_orig_price,
        &$has_default_rate_percentage_promo,
        &$has_default_rate_fixed_promo
    ) {
        $periods = $is_selected
            ? $acco['periods']
            : $acco['periods'][$first_price_key][0];
        $sorted = $is_selected ? $acco['sorted'] : 0;
        $data_prices = sprintf(
            'data-price=%s data-orig-price="%s"',
            $acco['price'],
            $acco['orig_price'] ?: ''
        );
        $total_rate_orig_price = $acco['orig_price'] ?: $acco['price'];
        if ($acco['orig_price']) {
            $has_default_rate_percentage_promo = true;
        }
        if ($acco['promo_amount']) {
            $has_default_rate_fixed_promo = true;
        }
        $rates_combined_list = '';
        $rate_ids = array();

        foreach ($periods as $period_part) {
            $rate_id = $period_part['rate_id'];
            $rate_ids[] = $rate_id;
            $rates_combined_list .= sprintf(
                '<li>%s %s (%s)</li>',
                OHBE_Tools::renderOccupancyIcons(
                    $rate_id,
                    'rates'
                ),
                isset($rates[$rate_id]['name'])
                    ? htmlspecialchars($rates[$rate_id]['name'])
                    : OHBE_Main::getInventory('rates')[$rate_id]['name'],
                sprintf(
                    _n('%s night', '%s nights', $period_part['days'], 'ohbe'),
                    $period_part['days']
                )
            );
        }

        return sprintf(
            '<div class=info data-amount=%s data-sorted=%s %s>'
            . '<div class=name>%s</div>'
            . '<div class="conditions modal-button" '
            . 'role=button data-modal=conditions-modal>'
            . '<a><span class="dashicons dashicons-info"></span> %s</a></div>'
            . '<ul class=combined-rates>%s</ul>',
            $acco['amount'],
            $sorted,
            $data_prices,
            __('Combination of rates', 'ohbe'),
            __('Conditions', 'ohbe'),
            $rates_combined_list
        );
    }

    /**
     * Render accommodation card's combined rate list block.
     *
     * @param  int $nights Nights.
     * @param  array $period_parts Rate period parts.
     * @param string $first_price_key First key of periods structure.
     * @param array $rates Translated rate names from Inventory.
     * @param string $rate_price Rate price.
     * @param string $sorted Order of the rate (used when there are more than one
     * with the same price).
     * @param string $best_price_tag Best price tag's html structure
     * @param string $promo_code Accommodation promotion code.
     * @param string $total_rate_orig_price Total rate original price.
     * @param float $promo_amount Discount promo amount price.
     * @param string $full_rate_structure Empty string to be filled and get all
     * rate ids in a row.
     * @param bool $has_rate_percentage_promo Tell if the rate has percentage-based
     * promo or not.
     * @param bool $has_rate_fixed_promo Tell if the rate has fixed-amount
     * promo or not.
     * @return string Accos-total block's html structure.
     *
     */
    public static function renderCombinedRateList(
        $nights,
        $period_parts,
        $first_price_key,
        $rates,
        $rate_price,
        $sorted,
        $best_price_tag,
        $promo_code,
        &$total_rate_orig_price,
        &$promo_amount,
        &$full_rate_structure,
        &$has_rate_percentage_promo,
        &$has_rate_fixed_promo
    ) {
        $rate_part = '';
        for ($i=0; $i<count($period_parts); $i++) {
            $rate_id = $period_parts[$i]['rate_id'];
            if ($i > 0) {
                $full_rate_structure .= ',';
            }
            $full_rate_structure .= $rate_id;
            if (isset($period_parts[$i]['orig_price'])) {
                $has_rate_percentage_promo = true;
            }
            if (isset($period_parts[$i]['promo_amount'])) {
                $has_rate_fixed_promo = true;
                $promo_amount += $period_parts[$i]['promo_amount'];
            }
            $total_rate_orig_price +=
                isset($period_parts[$i]['orig_price'])
                    ? $period_parts[$i]['orig_price']
                    : $period_parts[$i]['price'];
            $rate_part .= sprintf(
                '<li>%s (%s)</li>',
                isset($rates[$rate_id]['name'])
                    ? htmlspecialchars($rates[$rate_id]['name'])
                    : OHBE_Main::getInventory('rates')[$rate_id]['name'],
                $period_parts[$i]['night_string']
            );
        }
        $price = OHBE_Tools::formatPrice($total_rate_orig_price / $nights);
        $html = sprintf(
            '<div class=ohbe-title>%s %s</div>',
            OHBE_Tools::renderOccupancyIcons($period_parts[0]['rate_id'], 'rates'),
            __('Combination of rates', 'ohbe')
        );
        if ($has_rate_percentage_promo) {
            // Cross out the orig price when the promo is percentage-based
            $html .= '<div class="value orig">' . $price . '</div>';
        }
        elseif ($has_rate_fixed_promo) {
            // Set promo tag when the promo is fixed-amount
            $html .= '<div class="value promo"><div class="tag">'
                . $promo_code . '</div>' . $price . '</div>';
        }
        else {
            if ($first_price_key === $rate_price && $sorted === 0) {
                // Set best price tag when it is the cheapest rate
                $html .= '<div class="value promo">'
                    . $best_price_tag . $price . '</div>';
            }
            else {
                // Set only price
                $html .= '<div class="value">' . $price . '</div>';
            }
        }
        $html .= '<ul>' . $rate_part . '</ul>';

        return $html;
    }

    /**
     * Render the default accommodation rate adults & children data.
     * In other words, the adults & children data for a certain rate
     * when the page initializes.
     *
     * @param array $periods Accommodation periods
     * @return string
     */
    public static function renderDefaultAccommodationRateAdultsAndChildren(
        $periods
    ) {
        $first_price_key = current(array_keys($periods));

        return sprintf('data-adults="%s" data-children="%s"',
           $periods[$first_price_key][0][0]['adults'],
           $periods[$first_price_key][0][0]['children']
       );
    }

    /**
     * Render the default accommodation rate(s) ids when the page initializes.
     *
     * @param array $periods Accommodation periods
     * @return string Rate id(s). Example: 4,30,100.
     */
    public static function renderDefaultRatePartIds($periods) {
        $first_price_key = current(array_keys($periods));
        $html = '';
        for ($i=0; $i<count($periods[$first_price_key][0]); $i++) {
            if ($i > 0) {
                $html .= ',';
            }
            $html .= $periods[$first_price_key][0][$i]['rate_id'];
        }

        return $html;
    }

    /**
     * Render the accommodation card carousel.
     *
     * @param array $acco Single accommodation structure.
     * @param array $images_ids List of image ids.
     * @param bool $hasModal Tell if the carousel opens a modal.
     * @return string
     */
    public static function renderAccommodationCardCarousel(
        $acco,
        $images_ids,
        $hasModal
    ) {
        $carousel_id =
            "carousel-{$acco['acco_id']}-{$acco['arrival']}-{$acco['departure']}";
        $inv_images = OHBE_Main::getInventory('images');
        $html = '<div id='.$carousel_id.' class="carousel slide" data-interval=0>'
           . '<ol class=carousel-indicators>';
        for ($i = 0; $i < count($images_ids); $i++) {
            $html .= '<li data-target=#'.$carousel_id.' data-slide-to='. $i
                . ($i === 0 ? ' class=active' : '') .'></li>';
        }
        $html .= '</ol><div class=carousel-inner role=listbox>';
        for ($i = 0; $i < count($images_ids); $i++) {
            $image = '<img src='. $inv_images[$images_ids[$i]]['url'] .'>';
            $itemContent = $hasModal
                ? '<a class="description modal-button" '
                    . 'data-active-item=' . $images_ids[$i] . ' role=button '
                    . 'data-modal=description-modal>' . $image . '</a>'
                : $image;
            $isActive = $i === 0 ? 'active' : '';
            $html .= '<div class="item ' . $isActive . '">'
                . $itemContent . '</div>';
        }
        $html .= '</div>'
            . '<a '
                . 'class="left carousel-control" '
                . 'data-slide=prev '
                . 'href=#' . $carousel_id . ' '
                . 'role=button>'
            . '<span '
                . 'aria-hidden=true '
                . 'class="glyphicon glyphicon-chevron-left">'
            . '</span>'
            . '<span class=sr-only>Previous</span>'
            . '</a>'
            . '<a '
                . 'class="right carousel-control" '
                . 'href=#' . $carousel_id . ' '
                . 'role=button data-slide=next>'
            . '<span '
                . 'aria-hidden=true '
                . 'class="glyphicon glyphicon-chevron-right">'
            . '</span>'
            . '<span class=sr-only>Next</span>'
            . '</a>'
            . '</div>';

        return $html;
    }

    /**
     * Render accommodation card's image.
     *
     * @param  array $acco Single accommodation structure.
     * @return string Image's html structure.
     */
    public static function renderAccommodationCardImage($acco) {
        $inv_langs = OHBE_Main::getInventory('languages');
        $lang = OscarAPI::getLanguage();
        if (isset($inv_langs[$lang]['accos'])) {
            $inv_lang_accos = $inv_langs[$lang]['accos'];
        }
        $inv_accos = OHBE_Main::getInventory('accos');
        $hasModal = isset($inv_lang_accos[$acco['acco_id']]['description'])
            || count($inv_accos[$acco['acco_id']]['images']) > 0;
        if (
            isset($inv_accos[$acco['acco_id']]['images'])
            && count($inv_accos[$acco['acco_id']]['images']) > 1
        ) {
            $images_ids = $inv_accos[$acco['acco_id']]['images'];
            $html = self::renderAccommodationCardCarousel(
                $acco,
                $images_ids,
                $hasModal
            );
        } else {
            $image = OHBE_Tools::renderImage($acco['acco_id'], 'accos');
            $html = $hasModal
                ? '<a '
                    . 'class="description modal-button" '
                    . 'data-modal=description-modal '
                    . 'role=button>' . $image . '</a>'
                : $image;

        }

        return $html;
    }

    /**
     * Render accommodation card's name.
     *
     * @param  array $acco Single accommodation structure.
     * @return string Name's html structure.
     */
    public static function renderAccommodationCardName($acco) {
        $only_left = '';
        $lang = OscarAPI::getLanguage();
        $inv_accos = OHBE_Main::getInventory('accos');
        $inv_langs = OHBE_Main::getInventory('languages');
        $inv_lang_acco = isset($inv_langs[$lang]['accos'][$acco['acco_id']])
            ? $inv_langs[$lang]['accos'][$acco['acco_id']]
            : null;
        $name = isset($inv_lang_acco['name'])
            ? $inv_lang_acco['name']
            : $inv_accos[$acco['acco_id']]['name'];

        if (
            $acco['amount'] <= OHBE_Main::getSettings('only_left')
            && !isset($acco['promo_code'])
        ) {
            // Only left tag
            $only_left = sprintf(
                '<span class=only-left>%s</span>',
                sprintf(
                    _n('Only %d left', 'Only %d left', $acco['amount'], 'ohbe'),
                    number_format_i18n($acco['amount'])
                )
            );
        }

        if (isset($inv_lang_acco['description'])) {
            return sprintf(
                '<a class="description modal-button" '
                . 'role=button data-modal=description-modal title=%s>'
                . '%s<span class="dashicons dashicons-info"></span>'
                . '</a>%s',
                __('Description', 'ohbe'),
                $name,
                $only_left
            );
        }

        return "<span class=title>{$name}</span>{$only_left}";
    }

    /**
     * Render modals.
     *
     * @return string Modals' html structure.
     */
    public static function renderModals() {
        $inv_langs = OHBE_Main::getInventory('languages');
        $lang = OscarAPI::getLanguage();

        // Description
        $html = sprintf(
            '<div id=ohbe-description-modal class=ohbe-modal>'
            . '<div class=modal-dialog role=document>'
            . '<div class=modal-content>'
            . '<div class=modal-header>'
            . '<span class=close>&times;</span>'
            . '<h4 class=modal-title>%s</h4>'
            . '</div>'
            . '<div class=modal-body></div>'
            . '<div class=modal-footer>'
            . '<button type=button class="btn-ohbe btn-primary accept">%s</button>'
            . '</div></div></div></div>',
            __('Description', 'ohbe'),
            __('Close', 'ohbe')
        );
        // Conditions
        $html .= sprintf(
            '<div id=ohbe-conditions-modal class=ohbe-modal>'
            . '<div class=modal-dialog role=document>'
            . '<div class=modal-content>'
            . '<div class=modal-header>'
            . '<span class=close>&times;</span>'
            . '<h4 class="modal-title header">%s</h4>'
            . '</div>'
            . '<div class=modal-body>'
            . '<div class=conditions-content>',
            __('Reservation conditions', 'ohbe')
        );
        if (isset($inv_langs[$lang]['checkinout'])) {
            $html .= sprintf(
                '<h4 class=header>%s</h4>'
                . '<p class=time-text>%s</p>',
                __('Check in/out conditions', 'ohbe'),
                nl2br(htmlspecialchars($inv_langs[$lang]['checkinout']))
            );
        }
        if (isset($inv_langs[$lang]['conditions'])) {
            $html .= sprintf(
                '<h4 class=header>%s</h4>'
                . '<p class=conditions-text>%s</p>',
                __('Conditions', 'ohbe'),
                nl2br(htmlspecialchars($inv_langs[$lang]['conditions']))
            );
        }
        $html .= sprintf(
            '</div><div class=rate-content></div></div>'
            . '<div class=modal-footer>'
            . '<button type=button class="btn-ohbe btn-primary accept">%s</button>'
            . '</div></div></div></div>',
            __('Close', 'ohbe')
        );
        // Different dates
        $html .= sprintf(
            '<div id=ohbe-different-dates class=ohbe-modal>'
            . '<div class=modal-dialog role="document">'
            . '<div class=modal-content>'
            . '<div class=modal-header>'
            . '<span class=close>&times;</span>'
            . '<h4 class=modal-title>%s</h4>'
            . '</div>'
            . '<div class=modal-body>%s</div>'
            . '<div class=modal-footer>'
            . '<button type=button id=ohbe-different-dates-yes '
            . 'class="btn-ohbe btn-other-dates">%s</button>'
            . '<button type=button id=ohbe-different-dates-no '
            . 'class="btn-ohbe btn-other-dates">%s</button>'
            . '</div></div></div></div>',
            __('Confirm', 'ohbe'),
            __('You have already selected accommodations for other dates. '
            . 'Do you want to discard them?', 'ohbe'),
            __('Yes', 'ohbe'),
            __('No', 'ohbe')
        );

        return $html;
    }

    /**
     * Print promo ribbon when there is some rate with promotion.
     *
     * @param array $acco Single accommodation structure.
     * @return void
     */
    public static function printPromoRibbon($acco) {
        if (isset($acco['promo_code'])) {
            printf(
                '<div class=ribbon>'
                . '<span>%s</span></div>',
                __('Promo', 'ohbe')
            );
        }
    }

    /**
     * Render accommodation card's rate block.
     *
     * @param array $acco Single accommodation structure.
     * @return string Rate block's html structure.
     */
    public static function renderRateBlock($acco) {
        $inv_langs = OHBE_Main::getInventory('languages');
        $lang = OscarAPI::getLanguage();
        $inv_rates = isset($inv_langs[$lang]['rates'])
            ? $inv_langs[$lang]['rates']
            : OHBE_Main::getInventory('rates');
        $selected_acco = OHBE_Tools::getSelectedAcco($acco);
        $first_price_key = current(array_keys($acco['periods']));
        $total_rate_orig_price = 0;
        $has_default_rate_percentage_promo = false;
        $has_default_rate_fixed_promo = false;
        $best_price_tag = sprintf(
            '<div class="tag best-price">%s</div>',
            __('Best price', 'ohbe')
        );
        $html = '<div class=rate-block>';

        // Rate info
        if (count($acco['periods'][$first_price_key][0]) == 1) {
            // UNCOMBINED RATE
            $html .= OHBE_Availability::renderUncombinedRateInfo(
                $selected_acco ?: $acco,
                $selected_acco !== null,
                $inv_rates,
                $first_price_key,
                $total_rate_orig_price,
                $has_default_rate_percentage_promo,
                $has_default_rate_fixed_promo
            );
        }
        else {
            // COMBINED RATES
            $html .= OHBE_Availability::renderCombinedRateInfo(
                $selected_acco ?: $acco,
                $selected_acco !== null,
                $inv_rates,
                $first_price_key,
                $total_rate_orig_price,
                $has_default_rate_percentage_promo,
                $has_default_rate_fixed_promo
            );
        }

        $price = OHBE_Tools::formatPrice(
            floatval($total_rate_orig_price) / $acco['nights']
        );
        $html .= '<div class=price><div class=ohbe-title>'
            . __('Per night', 'ohbe') . ':</div>';
        if ($has_default_rate_percentage_promo) {
            // Cross out the orig price when the promo is percentage-based
            $html .= '<div class="value orig">' . $price . '</div>';
        }
        elseif (
            !$has_default_rate_percentage_promo
            && !$has_default_rate_fixed_promo
            && (!isset($selected_acco) || $selected_acco['price'] == $first_price_key)
        ) {
            // Set best price tag when it is the cheapest rate and has no promo
            $html .= '<div class="value">'
                . $best_price_tag . $price . '</div>';
        }
        else {
            // Set only price when no promo is set and it is not
            // the best price or fixed-amount promo
            $html .= '<div class="value">' . $price . '</div>';
        }
        $html .= '</div>';
        if ($has_default_rate_percentage_promo) {
            // Print a second line when there is a percentage-based promo
            $price = floatval(
                isset($selected_acco)
                    ? $selected_acco['price']
                    : $first_price_key
            );
            $html .= sprintf(
                '<div class=promo>'
                . '<div class="ohbe-title tag">%s</div>'
                . '<div class=value>%s</div></div>',
                $acco['promo_code'],
                OHBE_Tools::formatPrice($price / $acco['nights'])
            );
        }

        // Rates list
        $full_rate_structure = $rate_list_final = '';
        $rate_elements_quantity = 0;

        foreach ($acco['periods'] as $rate_price => $sorted_period) {
            foreach ($sorted_period as $sorted => $period_parts) {
                $data_promo_amount = $data_orig_price = '';
                $promo_amount = $total_rate_orig_price = 0;
                $is_combined_rate
                    = $has_rate_percentage_promo
                    = $has_rate_fixed_promo
                    = false;
                if (count($period_parts) == 1) {
                    // UNCOMBINED RATE
                    $rate_list = OHBE_Availability::renderUncombinedRateList(
                        $acco['nights'],
                        $period_parts[0],
                        $first_price_key,
                        $inv_rates,
                        $rate_price,
                        $sorted,
                        $best_price_tag,
                        $total_rate_orig_price,
                        $promo_amount,
                        $full_rate_structure,
                        $has_rate_percentage_promo,
                        $has_rate_fixed_promo
                    );
                }
                else {
                    // COMBINED RATES
                    $rate_list = OHBE_Availability::renderCombinedRateList(
                        $acco['nights'],
                        $period_parts,
                        $first_price_key,
                        $inv_rates,
                        $rate_price,
                        $sorted,
                        $best_price_tag,
                        $acco['promo_code'],
                        $total_rate_orig_price,
                        $promo_amount,
                        $full_rate_structure,
                        $has_rate_percentage_promo,
                        $has_rate_fixed_promo
                    );
                    $is_combined_rate = true;
                }

                if ($has_rate_percentage_promo) {
                    $data_orig_price = ' data-orig-price='. $total_rate_orig_price;
                }
                if ($promo_amount > 0) {
                    $data_promo_amount = ' data-promo-amount=' . $promo_amount;
                }
                if (
                    $has_rate_percentage_promo
                    || ($has_rate_fixed_promo && !$is_combined_rate)
                ) {
                    // Do not print the ribbon when rate is combined and
                    // rate has fixed promo
                    $rate_list .= sprintf(
                        '<div class=total-promo>'
                        . '<div class="ohbe-title tag">%s</div>'
                        . '<div class=value>%s</div>'
                        . '</div>',
                        $acco['promo_code'],
                        OHBE_Tools::formatPrice($rate_price / $acco['nights'])
                    );
                }

                if ($selected_acco) {
                    $show_selected = $selected_acco['rate'] == $full_rate_structure
                        ? ' selected' : '';
                }
                else {
                    $show_selected = $rate_price == $first_price_key && $sorted == 0
                        ? ' selected' : '';
                }
                $rate_list_final .= sprintf(
                    '<li class="rate%s" data-price="%s"%s%s '
                    . 'data-sorted=%s data-adults=%s data-children=%s>%s',
                    $show_selected,
                    $rate_price,
                    $data_orig_price,
                    $data_promo_amount,
                    $sorted,
                    $period_parts[0]['adults'],
                    $period_parts[0]['children'],
                    $rate_list
                );
                $rate_elements_quantity++;
            }
            $rate_list_final .= '</li>';
        }

        $html .= '</div>';
        // Print / Hide rate selection
        $html .= sprintf(
            '<button class=btn-select-rate%s>%s'
            . '<span class="dashicons dashicons-arrow-down"></span></button>'
            . '<ul class=rate-list>%s</ul>',
            $rate_elements_quantity > 1 ? '' : ' style="visibility: hidden;"',
            __('Show other options', 'ohbe'),
            $rate_list_final
        );
        $html .= sprintf(
            '%s</div></div>',
            OHBE_Availability::renderAccosTotalBlock($acco)
        );

        return $html;
    }

    /**
     * Render accommodation card's combined rate info block.
     *
     * @param  array $acco Single accommodation structure.
     * @param  bool $is_selected Tell if the accommodation has been selected
     * previously.
     * @param array $rates Translated rate names from Inventory.
     * @param string $first_price_key First key of periods structure.
     * @param string $total_rate_orig_price Total rate original price.
     * @param bool $has_default_rate_percentage_promo Tell if the rate has
     * percentage-based promo or not.
     * @param bool $has_default_rate_fixed_promo Tell if the rate has fixed-amount
     * promo or not.
     * @return string Accos-total block's html structure.
     *
     */
    public static function renderUncombinedRateInfo(
        $acco,
        $is_selected,
        $rates,
        $first_price_key,
        &$total_rate_orig_price,
        &$has_default_rate_percentage_promo,
        &$has_default_rate_fixed_promo
    ) {
        $rate_id = $is_selected
        ? $acco['rate']
        : $acco['periods'][$first_price_key][0][0]['rate_id'];
        $sorted = $is_selected ? $acco['sorted'] : 0;
        $data_prices = sprintf(
            'data-price=%s data-orig-price="%s"',
            $acco['price'],
            $acco['orig_price'] ?: ''
        );
        $total_rate_orig_price = $acco['orig_price'] ?: $acco['price'];
        if ($acco['orig_price']) {
            $has_default_rate_percentage_promo = true;
        }
        if ($acco['promo_amount']) {
            $has_default_rate_fixed_promo = true;
        }
        $html = sprintf(
            '<div class=info data-amount=%s data-sorted=%s %s>',
            $acco['amount'],
            $sorted,
            $data_prices
        );
        $html .= sprintf(
            '<div class=name>%s %s</div>'
            . '<div class="conditions modal-button" '
            . 'role=button data-modal=conditions-modal>'
            . '<a><span class="dashicons dashicons-info"></span> %s</a></div>',
            OHBE_Tools::renderOccupancyIcons($rate_id, 'rates'),
            isset($rates[$rate_id]['name'])
                ? htmlspecialchars($rates[$rate_id]['name'])
                : OHBE_Main::getInventory('rates')[$rate_id]['name'],
            __('Conditions', 'ohbe')
        );

        return $html;
    }

    /**
     * Render accommodation card's combined rate list block.
     *
     * @param  int $nights Nights.
     * @param  array $rate Rate single period part.
     * @param string $first_price_key First key of periods structure.
     * @param array $rates Translated rate names from Inventory.
     * @param string $rate_price Rate price.
     * @param string $sorted Order of the rate (used when there are more than one
     * with the same price).
     * @param string $best_price_tag Best price tag's html structure
     * @param float $total_rate_orig_price Total rate original price.
     * @param float $promo_amount Discount promo amount price.
     * @param string $full_rate_structure Empty string to be filled and get all
     * rate ids in a row
     * @param bool $has_rate_percentage_promo Tell if the rate has percentage-based
     * promo or not.
     * @param bool $has_rate_fixed_promo Tell if the rate has fixed-amount
     * promo or not.
     * @return string Accos-total block's html structure.
     *
     */
    public static function renderUncombinedRateList(
        $nights,
        $rate,
        $first_price_key,
        $rates,
        $rate_price,
        $sorted,
        $best_price_tag,
        &$total_rate_orig_price,
        &$promo_amount,
        &$full_rate_structure,
        &$has_rate_percentage_promo,
        &$has_rate_fixed_promo
    ) {
        $rate_id = $rate['rate_id'];
        $full_rate_structure = $rate['rate_id'];
        $has_rate_percentage_promo = isset($rate['orig_price']);
        $has_rate_fixed_promo = isset($rate['promo_amount']);
        if (isset($rate['promo_amount']))
            $promo_amount = $rate['promo_amount'];
        $total_rate_orig_price = isset($rate['orig_price'])
            ? $rate['orig_price'] : $rate['price'];

        $html = sprintf(
            '<div class=ohbe-title>%s %s</div>',
            OHBE_Tools::renderOccupancyIcons($rate_id, 'rates'),
            isset($rates[$rate_id]['name'])
                ? htmlspecialchars($rates[$rate_id]['name'])
                : OHBE_Main::getInventory('rates')[$rate_id]['name']
        );

        if (!isset($rate['promo_amount'])) {
            $html .= sprintf(
                '<div class="value%s">%s %s</div>',
                isset($rate['orig_price']) ? ' orig' : '',
                (!$has_rate_percentage_promo && !$has_rate_fixed_promo)
                && $first_price_key == $rate_price
                && $sorted == 0 ? $best_price_tag : '',
                OHBE_Tools::formatPrice(floatval($total_rate_orig_price) / $nights)
            );
        }

        return $html;
    }
}
