<?php
defined('ABSPATH') or exit();

/**
 * Redforts Oscar Hotel Booking
 * Reservation functions
 *
 * @author Redforts Software S.L.
 * @copyright Copyright (c) 2015 Redforts Software S.L.
 * @link https://redforts.com
 * @since 0.1.1
 */
class OHBE_Reservation
{
    /**
     * Delete saved reservation params.
     *
     * @return null
     */
    public static function deleteReservationParam() {
        unset($_SESSION['ohbe']['reservation']);
    }

    /**
     * Get translated name label.
     *
     * @param string $field_id Name of the field
     * @return string Name of label.
     */
    public static function getFieldLabel($field_id) {
        switch ($field_id) {
            case 'firstnames':
                return __('First names', 'ohbe');
            case 'middlenames':
                return __('Middle names', 'ohbe');
            case 'surname':
                return __('Surname', 'ohbe');
            case 'surname2':
                return __('Surname 2', 'ohbe');
            case 'address':
                return __('Address', 'ohbe');
            case 'town':
                return __('Town', 'ohbe');
            case 'state':
                return __('State', 'ohbe');
            case 'postalcode':
                return __('Postal code', 'ohbe');
            case 'phone':
                return __('Phone', 'ohbe');
            case 'phone2':
                return __('Phone 2', 'ohbe');
            case 'email':
                return __('Email', 'ohbe');
            case 'country':
                return __('Country', 'ohbe');
            case 'arrival_time':
                return __('Arrival time', 'ohbe');
            case 'departure_time':
                return __('Departure time', 'ohbe');
            case 'card_issuer':
                return __('Issuer', 'ohbe');
            case 'card_number':
                return __('Number', 'ohbe');
            case 'card_csc':
                return __('CSC', 'ohbe');
            case 'card_holder':
                return __('Holder', 'ohbe');
            case 'card_expires':
                return __('Expiry', 'ohbe');
        }
    }

    /**
     * Set accommodation structure for Oscar reservation call.
     *
     * @return array Accommodations list ready for API call.
     */
    public static function getAccommodationStructure() {
        $accos = array();

        if (isset($_SESSION['ohbe']['selected_accos'])) {
            foreach ($_SESSION['ohbe']['selected_accos'] as $sa) {
                for ($i=0; $i<$sa['quantity']; $i++) {
                    $acco = array(
                        'adults' => intval(
                            $_POST['ohbe_adults_' . $sa['acco_id'] . '_' . $i]
                        ),
                        'acco_id' => intval($sa['acco_id']),
                        'price' => $sa['price'],
                        'periods' => [],
                    );
                    if (isset($_POST['ohbe_children_' . $sa['acco_id'] . '_' . $i]))
                        $acco['children'] = intval(
                            $_POST['ohbe_children_' . $sa['acco_id'] . '_' . $i]
                        );
                    foreach ($sa['periods'] as $p) {
                        $acco['periods'][] = array(
                            'rate_id' => intval($p['rate_id']),
                            'days' => intval($p['days']),
                        );
                    }
                    $accos[] = $acco;
                }
            }
        }

        return $accos;
    }

    /**
     * Get the values of every field when a reservation has been made.
     *
     * @return array|string List of reservation params or string error.
     */
    public static function getReservationFieldValues() {
        $reservation_data = array();
        foreach ($_POST as $key => $value) {
            // Add only values that contains 'ohbe_'
            // at the beginning of the string
            if (strpos($key, 'ohbe_') === 0) {
                // Accommodations sanitization
                if (is_array($value)) {
                    for ($i=0; $i<count($value); $i++) {
                        $reservation_data[$key][$i] = $value[$i];
                    }
                }
                else {
                    // Fields sanitization
                    $reservation_data[$key] = $value;
                }
            }
        }
        if (
            array_key_exists('ohbe_card_expires_month', $reservation_data)
            && array_key_exists('ohbe_card_expires_year', $reservation_data)
        ) {
            $card_expires_date = DateTime::createFromFormat(
                'Y-m-d',
                $reservation_data['ohbe_card_expires_year']
                . '-'
                . $reservation_data['ohbe_card_expires_month']
                . '-01'
            );
            $card_expires_date->modify('+1 month');
            $card_expires_date->modify('-1 day');
            $reservation_data['ohbe_card_expires'] = $card_expires_date->format(
                'Y-m-d'
            );
        }

        return $reservation_data;
    }

    /**
     * Get reservation full name.
     *
     * @param array $reservation_data Reservation data.
     * @return string
     */
    public static function getReservationFullName($reservation_data) {
        $fullName = '';

        if (isset($reservation_data['ohbe_firstnames'])) {
            $fullName .= $reservation_data['ohbe_firstnames'] . ' ';
        }
        if (isset($reservation_data['ohbe_middlenames'])) {
            $fullName .= $reservation_data['ohbe_middlenames'] . ' ';
        }
        if (isset($reservation_data['ohbe_surname'])) {
            $fullName .= $reservation_data['ohbe_surname'] . ' ';
        }
        if (isset($reservation_data['ohbe_surname2'])) {
            $fullName .= $reservation_data['ohbe_surname2'];
        }

        return trim($fullName);
    }

    /**
     * Get the param/s for saved reservation.
     *
     * @param mixed $param A single reservation parameter
     * @return array|null
     */
    public static function getReservationParam($param=null) {
        if (isset($_SESSION['ohbe']['reservation'])) {
            $reservation = $_SESSION['ohbe']['reservation'];
            return isset($param) ? $reservation[$param] : $reservation;
        }

        return null;
    }

    /**
     * Handle API error for the Reservation call.
     * Only get the first error in the array returned by the API call.
     * Depends on the type of error, a redirection might be needed.
     *
     * @param array $error API Error.
     * @return boolean Whether a redirection is needed.
     */
    public static function handleReservationError($error) {
        $isRedirect = false;
        switch($error['code']) {
            case 200:
            case 201:
            case 202:
                $errorMessage = __(
                    'Incorrect information in highlighted field',
                    'ohbe'
                );
                foreach ($error['values'] as $value) {
                    OHBE_Tools::setApiErrors($value, true);
                }
                break;
            default:
                $errorMessage = $error['text'];
                OHBE_Tools::removeSelectedAccos();
                $isRedirect = true;
                break;
        }
        OHBE_Tools::setFlashMessage($errorMessage, 'danger');

        return $isRedirect;
    }

    /**
     * Print credit card header when some credit card field
     * is set in inventory.
     *
     * @return void
     */
    public static function printCreditCardDataHeader() {
        $fields = OHBE_Main::getInventory('fields');

        foreach ($fields as $field) {
            if (strncmp($field['id'], "card", 4) == 0) {
                printf(
                    '<h3 class="ohbe-title">%s</h3>',
                    __('Credit card data', 'ohbe')
                );
                return;
            }
        }
    }

    /**
     * Render a guest field.
     *
     * @param integer $index Number of accommodation's quantity
     * @param integer $acco_id Accommodation's id.
     * @param integer $rate_id Rate's id.
     * @param integer $type Type of guest.
     * @return string
     */
    public static function renderGuestsNumberField(
        $index,
        $acco_id,
        $rate_id,
        $type
    ) {
        $field_name = 'ohbe_' . $type . '_' . $acco_id . '_' . $index;
        $guests = OHBE_Tools::getAccoGuests($acco_id, $rate_id);
        $min = 1;
        $max = $guests['adults'];
        if ($type == 'children') {
            $guests = OHBE_Tools::getAccoGuests($acco_id, $rate_id);
            $min = 0;
            $max = $guests['adults'] + $guests['children'] - 1;
        }
        $display_value = isset($_POST[$field_name])
            ? $_POST[$field_name]
            : $guests[$type];
        $html = ' <select name=' . $field_name
            . ' class="ohbe-select ohbe-field middle people-number">';
        for ($i = $min; $i <= $max; $i++) {
            $value_text = $type == 'adults'
                ? sprintf(_n('%s adult', '%s adults', $i, 'ohbe'), $i)
                : sprintf(_n('%s child', '%s children', $i, 'ohbe'), $i);
            $html .= sprintf(
                '<option value="%s"%s>%s</option>',
                $i,
                $display_value == $i ? ' selected' : '',
                $value_text
            );
        }
        $html .= '</select>';

        return $html;
    }

    /**
     * Print a block of number of persons fields.
     *
     * @return void
     */
    public static function printGuestsNumberFields() {
        if (isset($_SESSION['ohbe']['selected_accos'])) {
            echo '<h3 class=ohbe-title>' . __('Number of persons', 'ohbe') . '</h3>';
            $html = '';

            foreach ($_SESSION['ohbe']['selected_accos'] as $sa) {
                for ($i = 0; $i < $sa['quantity']; $i++) {
                    // Set fields group
                    $html .= sprintf(
                        '<div class="reservation-field" data-max="%s">',
                        $sa['adults'] + $sa['children']
                        )
                        . '<label>'
                        . OHBE_Tools::getAccoName($sa['acco_id'])
                        . ' '
                        . OHBE_Tools::renderOccupancyIcons($sa['rate'], 'rates')
                        . '</label>';
                    // Set adults
                    $html .= self::renderGuestsNumberField(
                        $i,
                        $sa['acco_id'],
                        $sa['rate'],
                        'adults'
                    );
                    // Set children
                    if (OHBE_Main::getInventory('allow_children')) {
                        $html .= self::renderGuestsNumberField(
                            $i,
                            $sa['acco_id'],
                            $sa['rate'],
                            'children'
                        );
                    }
                    $html .= '</div>';
                }
            }

            echo $html;
        }
    }

    /**
     * Print the rate(s) occupancy icons
     *
     * @param bool $isRemovable Tell when the accommodation card is removable.
     * @return void
     */
    public static function printRemovable($isRemovable) {
        if ($isRemovable) {
            print(
                '<div class="delete"><a class="delete-acco" role="button">'
                . '<span class="dashicons dashicons-no"></span></a></div>'
            );
        }
    }

    /**
     * Print a block of fields.
     *
     * @param string $data_type Block type.
     * @return void
     */
    public static function printReservationFields($data_type) {
        $fields = OHBE_Main::getInventory('fields');
        foreach ($fields as $field) {
            self::printReservationFieldType($field, $data_type);
        }
    }

    /**
     * Print field.
     *
     * @param array $field Inventory field element.
     * @param string $data_type Block type.
     * @return void
     */
    public static function printReservationFieldType($field, $data_type) {
        $label = self::getFieldLabel($field['id']);

        switch($data_type) {
            case 'personal':
                switch ($field['id']) {
                    case 'firstnames':
                    case 'middlenames':
                    case 'surname':
                    case 'surname2':
                    case 'address':
                    case 'town':
                    case 'state':
                        printf(
                            '<div class=reservation-field>'
                            . '<label for=%s>%s%s</label>%s</div>',
                            'ohbe_' . htmlspecialchars($field['id']),
                            $label,
                            $field['required']
                                ? '<span class=ohbe-field-required>*</span>' : '',
                            OHBE_Reservation::renderReservationFieldInput($field)
                        );
                        break;
                    case 'postalcode':
                    case 'phone':
                    case 'phone2':
                        printf(
                            '<div class=reservation-field>'
                            . '<label for=%s>%s%s</label>%s</div>',
                            'ohbe_' . htmlspecialchars($field['id']),
                            $label,
                            $field['required']
                                ? '<span class=ohbe-field-required>*</span>' : '',
                            OHBE_Reservation::renderReservationFieldInput(
                                $field,
                                'tel'
                            )
                        );
                        break;
                    case 'email':
                        printf(
                            '<div class=reservation-field>'
                            . '<label for=%s>%s%s</label>%s</div>',
                            'ohbe_' . htmlspecialchars($field['id']),
                            $label,
                            $field['required']
                                ? '<span class=ohbe-field-required>*</span>' : '',
                            OHBE_Reservation::renderReservationFieldInput(
                                $field,
                                'email'
                            )
                        );
                        break;
                    case 'arrival_time':
                    case 'country':
                    case 'departure_time':
                        printf(
                            '<div class=reservation-field>'
                            . '<label for=%s>%s%s</label>%s</div>',
                            'ohbe_' . htmlspecialchars($field['id']),
                            $label,
                            $field['required']
                                ? '<span class=ohbe-field-required>*</span>' : '',
                            OHBE_Reservation::renderReservationFieldDropdown($field)
                        );
                        break;
                }
                break;

            case 'card':
                switch ($field['id']) {
                    case 'card_issuer':
                        if (
                            OHBE_Main::getInventory('creditcard_issuers')
                            && count(OHBE_Main::getInventory('creditcard_issuers')) > 0
                        ) {
                            printf(
                                '<div class=reservation-field>'
                                . '<label for=%s>%s%s</label>%s</div>',
                                'ohbe_' . htmlspecialchars($field['id']),
                                $label,
                                $field['required']
                                    ? '<span class=ohbe-field-required>*</span>' : '',
                                OHBE_Reservation::renderReservationFieldDropdown(
                                    $field
                                )
                            );
                        }
                        break;
                    case 'card_number':
                        printf(
                            '<div class=reservation-field>'
                            . '<label for=%s>%s%s</label>%s</div>',
                            'ohbe_' . htmlspecialchars($field['id']),
                            $label,
                            $field['required']
                                ? '<span class=ohbe-field-required>*</span>' : '',
                            OHBE_Reservation::renderReservationFieldInput(
                                $field,
                                'tel',
                                'card_number'
                            )
                        );
                        break;
                    case 'card_csc':
                        printf(
                            '<div class=reservation-field>'
                            . '<label for=%s>%s%s</label>%s</div>',
                            'ohbe_' . htmlspecialchars($field['id']),
                            $label,
                            $field['required']
                                ? '<span class=ohbe-field-required>*</span>' : '',
                            OHBE_Reservation::renderReservationFieldInput(
                                $field,
                                'tel',
                                'card_csc'
                            )
                        );
                        break;
                    case 'card_holder':
                        printf(
                            '<div class=reservation-field>'
                            . '<label for=%s>%s%s</label>%s</div>',
                            'ohbe_' . htmlspecialchars($field['id']),
                            $label,
                            $field['required']
                                ? '<span class=ohbe-field-required>*</span>' : '',
                            OHBE_Reservation::renderReservationFieldInput($field)
                        );
                        break;
                    case 'card_expires':
                        printf(
                            '<div class=reservation-field>'
                            . '<label for=%s>%s%s</label>%s %s</div>',
                            'ohbe_' . htmlspecialchars($field['id']) . '_month',
                            $label,
                            $field['required']
                                ? '<span class=ohbe-field-required>*</span>' : '',
                            OHBE_Reservation::renderReservationFieldDropdown(
                                $field,
                                '_month'
                            ),
                            OHBE_Reservation::renderReservationFieldDropdown(
                                $field,
                                '_year'
                            )
                        );
                        break;
                }
                break;
        }
    }

    /**
     * Render the accommodations selected in the availability page.
     * Display every accommodation cards.
     *
     * @param int $total_price Accommodations' total price.
     * @return void
     */
    public static function printSelectedAccommodations(&$total_price) {
        $selected_accommodations = array();
        $promo_amount_total = 0;

        // Reservation
        if (
            $_SERVER['REQUEST_METHOD'] == "POST"
            && isset($_POST['ohbe_go_reservation_page'])
            && isset($_POST['accommodations'])
        ) {
            $accommodations_exchangeable = json_decode(
                html_entity_decode(stripslashes($_POST['accommodations'])),
                true
            );
            if (isset($_POST['promo_total'])) {
                $promo_total = floatval($_POST['promo_total']);
            }
        }
        else if (
            isset($_SESSION['ohbe']['selected_accos'])
            && !empty($_SESSION['ohbe']['selected_accos'])
        ) {
            $accommodations_exchangeable = $_SESSION['ohbe']['selected_accos'];
            if (isset($_SESSION['ohbe']['promo-total'])) {
                $promo_total = floatval($_SESSION['ohbe']['promo_total']);
            }
        } else {
            OHBE_Tools::setFlashMessage(
                __(
                    'There are no accommodations selected. Please, try again.',
                    'ohbe'
                ),
                'danger'
            );
        }

        $isRemovable = false;

        OHBE_Tools::printFlashMessage();
        echo '<h3 class=ohbe-selected-list-title>'
            . __('Selected accommodations', 'ohbe')
            . '</h3><form method=POST action='
            . OHBE_Tools::renderFormAction(OHBE_RESERVATION_PAGE, true) .'>'
            . '<div class=booking-confirmation-summary>'
            . '<ul class=accommodation-booked-list>';

        if (isset($accommodations_exchangeable)) {
            $_SESSION['ohbe']['selected_accos'] = array();
            foreach ($accommodations_exchangeable as $acco) {
                $_SESSION['ohbe']['selected_accos'][] = $acco;
                $selected_accommodations[] = $acco;
                // Sum all accommodations promo_amount
                if ($acco['promo_amount']) {
                    $promo_amount_total +=
                        $acco['quantity'] * floatval($acco['promo_amount']);
                }
                $total_price += $acco['quantity'] * $acco['price'];
                include(
                    OHBE_PATH . 'includes/templates/views/'
                    . 'accommodation_reservation_card.php'
                );
            }
        }

        writeLog(
            session_id() . ' '
            . __CLASS__ . ' '
            . __FUNCTION__ . ' '
            . json_encode($_SESSION['ohbe']['selected_accos'])
        );

        echo '</ul></div>';

        // Set orig_price and new total_price
        if (isset($promo_total) && $promo_amount_total > 0) {
            $orig_price = $total_price;
            $total_price -= min($promo_amount_total, $promo_total);
            if ($total_price < 0) {
                $total_price = 0;
            }
        }

        OHBE_Tools::printBookedListBottom(
            $total_price,
            'modify-btn',
            'modify_submit',
            '<span class="dashicons dashicons-undo"></span>'. __('Modify', 'ohbe'),
            false,
            isset($orig_price) ? $orig_price : null
        );
        // TODO: Element is repeated.
        // echo '<input type=hidden id=ohbe-accommodations-input name=accommodations>'
          echo '</form>';
    }

    /**
     * Render accommodation card's name.
     *
     * @param array $acco Single accommodation structure.
     * @return string
     */
    public static function renderAccommodationCardName($acco) {
        $quantity = '';
        if ($acco['quantity'] > 1) {
            $quantity = sprintf('%s × ', $acco['quantity']);
        }

        return $quantity . $acco['name'];
    }

    /**
     * Render comments.
     *
     * @param array $inventory_language Inventory language data.
     * @return string Comment's html structure.
     */
    public static function renderComments($inventory_language) {
        $html = '';

        if (OHBE_Main::getSettings('notes')) {
            $html .= '<h3 class=ohbe-title>'
                .  __('Comments or requests', 'ohbe')
                . '</h3>';
            if (isset($inventory_language['note_text'])) {
                $html .= htmlspecialchars($inventory_language['note_text']);
            }
            $html .= '<div class="reservation-field full-width">'
                . '<div class=ohbe-comments>'
                . '<textarea id=ohbe-notes class=ohbe-field name=ohbe_notes>'
                . '</textarea></div></div>';
        }

        return $html;
    }

    /**
     * Render conditions checkbox.
     *
     * @param array $inventory_language Inventory language data.
     * @return string Checkbox's html structure.
     */
    public static function renderConditions($inventory_language) {
        $html = '';

        if (
            OHBE_Main::getSettings('conditions')
            && isset($inventory_language)
            && (
                isset($inventory_language['checkinout'])
                || isset($inventory_language['conditions'])
            )
        ) {
            $link = sprintf(__(
                    'I have read the <a %s>reservation conditions</a>'
                    . ' and I accept it.'
                    , 'ohbe'
                ),
                'class="modal-button" role="button" data-modal=conditions-modal'
            );
            $html = '<div class=ohbe-conditions>'
                . '<h3 class=ohbe-title>' . __('Conditions', 'ohbe') . '</h3>'
                . '<input type=checkbox name=accept_conditions required> '
                . $link
                . '<span class=ohbe-field-required>*</span>'
                . '</div>';
        }

        return $html;
    }

    /**
     * Render rate conditions.
     *
     * @return string Rate conditions html structure.
     */
    public static function renderConditionsRateContent() {
        $html = '';

        if (isset($_SESSION['ohbe']['selected_accos'])) {
            $inv_langs = OHBE_Main::getInventory('languages');
            $inv_rates = OHBE_Main::getInventory('rates');
            $lang = OscarAPI::getLanguage();
            if (isset($inv_langs[$lang]['rates'])) {
                $inv_lang_rates = $inv_langs[$lang]['rates'];
            }
            $rate_ids = array();
            $html = '<div class=rate-content>';

            foreach ($_SESSION['ohbe']['selected_accos'] as $acco) {
                $acco_rate_ids = explode(',', $acco['rate']);

                foreach ($acco_rate_ids as $id) {
                    if (!in_array($id, $rate_ids)) {
                        $rate_ids[] = $id;
                        if (isset($inv_lang_rates[$id])) {
                            $inv_lang_rate = $inv_lang_rates[$id];
                        }
                        $name = isset($inv_lang_rate['name'])
                            ? $inv_lang_rate['name']
                            : $inv_rates[$id]['name'];
                        $html .= '<div class=rate-header>'
                            . OHBE_Tools::renderOccupancyIcons($id, 'rates')
                            . '<h4 class=header>' . htmlspecialchars($name)
                            . '</h4></div>';
                        if (isset($inv_lang_rate['description'])) {
                            $sanitised_description = nl2br(
                                htmlspecialchars($inv_lang_rate['description'])
                            );
                            $html .= '<h4 class=header>' . __('Description', 'ohbe')
                                . '</h4><p>' . $sanitised_description . '</p>';
                        }
                        if (isset($inv_lang_rate['payment'])) {
                            $sanitised_payment = nl2br(
                                htmlspecialchars($inv_lang_rate['payment'])
                            );
                            $html .= '<h4 class=header>' . __('Payment policy', 'ohbe')
                                . '</h4><p>' . $sanitised_payment . '</p>';
                        }
                        if (isset($inv_lang_rate['cancellation'])) {
                            $sanitised_cancellation = nl2br(
                                htmlspecialchars($inv_lang_rate['cancellation'])
                            );
                            $html .= '<h4 class=header>' . __('Cancellation policy', 'ohbe')
                                . '</h4><p>' . $sanitised_cancellation . '</p>';
                        }
                    }
                }
            }
            $html .= '</div>';
        }

        return $html;
    }

    /**
     * Render consent block.
     *
     * @param array $inventory_language Inventory language data.
     * @return string Consent block html structure.
     */
    public static function renderConsent($inventory_language) {
        $html = '';

        if (
            isset($inventory_language)
            && isset($inventory_language['consent_title'])
            && isset($inventory_language['consent_text'])
        ) {
            $html .= sprintf(
                '<h3 class=ohbe-title>%s</h3>'
                . '<label for=ohbe_consent class=ohbe-checkbox-label>'
                . '<input id=ohbe_consent type=checkbox name=ohbe_consent> %s'
                . '</label>',
                htmlspecialchars($inventory_language['consent_title']),
                nl2br($inventory_language['consent_text'])
            );
        }

        return $html;
    }

    /**
     * Render modal.
     *
     * @param string $id Name of the modal.
     * @param string $title Translated title.
     * @param string $content Modal's content.
     * @return string Modal's html structure.
     */
    public static function renderModal($id, $title, $content) {
        return sprintf(
            '<div id=%s-modal class=ohbe-modal>'
            . '<div class=modal-dialog role=document>'
            . '<div class=modal-content>'
            . '<div class=modal-header>'
            . '<span class=close>&times;</span>'
            . '<h4 class=modal-title>%s</h4>'
            . '</div>'
            . '<div class=modal-body>%s%s</div><div class=modal-footer>'
            . '<button type=button class="btn-ohbe btn-modal accept">%s</button>'
            . '</div></div></div></div>',
            $id,
            $title,
            $content,
            $id == 'ohbe-conditions'
                ? OHBE_Reservation::renderConditionsRateContent()
                : '',
            __('Close', 'ohbe')
        );
    }

    /**
     * Render privacy statement block.
     *
     * @param array $inventory_language Inventory language data.
     * @return string Privacy statement block html structure.
     */
    public static function renderPrivacyStatement($inventory_language) {
        $html = '';

        if (
            isset($inventory_language)
            && (
                isset($inventory_language['privacy'])
                || isset($inventory_language['privacy_summary'])
            )
        ) {
            $html .= '<h3 class=ohbe-title>'
                . __('Privacy statement', 'ohbe')
                . '</h3>';
        }

        if (isset($inventory_language['privacy_summary'])) {
            $html .= nl2br(htmlspecialchars($inventory_language['privacy_summary']))
                . ' ';
        }

        if (isset($inventory_language['privacy'])) {
            $html .= '<span class=ohbe-privacy>'
                . sprintf(
                    __(
                        'For more information please consult the '
                        . '<a %s>privacy statement</a>'
                        , 'ohbe'
                    ),
                    'class=modal-button role=button data-modal=privacy-modal'
                )
                . '</span>';
        }

        return $html;
    }

    /**
     * Render the name(s) of rate(s)
     *
     * @param array $periods Accommodation periods structure.
     * @return string Name(s) of rate(s).
     */
    public static function renderRateNames($periods) {
        $inv_langs = OHBE_Main::getInventory('languages');
        $lang = OscarAPI::getLanguage();
        $rate_names = array();

        foreach ($periods as $p) {
            $rate_names[] = isset($inv_langs[$lang]['rates'][$p['rate_id']]['name'])
                ? $inv_langs[$lang]['rates'][$p['rate_id']]['name']
                : OHBE_Main::getInventory('rates')[$p['rate_id']]['name'];
        }

        return htmlspecialchars(join(', ', $rate_names));
    }

    /**
     * Render the rate(s) occupancy icons
     *
     * @param array $acco Accommodation.
     * @return string Name(s) of rate(s).
     */
    public static function renderRateOccupancyIcons($acco) {
        $inventory_rates = OHBE_Main::getInventory('rates');
        $html = '';

        foreach ($acco['periods'] as $period) {
            if (array_key_exists($period['rate_id'], $inventory_rates)) {
                $html .= OHBE_Tools::renderOccupancyIcons(
                    $period['rate_id'],
                    'rates'
                );
                break;
            }
        }
        if ($html == '') {
            $html .= OHBE_Tools::renderOccupancyIcons($acco['acco_id'], 'accos');
        }

        return $html;
    }

    /**
     * Render dropdown field.
     *
     * @param array $field Inventory field element.
     * @param string $extra_data Used for differentiation (_month|_year).
     * Those are used for 'card_expires' values.
     * @return string Dropdown's html structure.
     */
    private static function renderReservationFieldDropdown($field, $extra_data='') {
        $is_error = OHBE_Tools::hasAPIErrors($field['id']);
        $error_border = $is_error ? ' error-border' : '';
        $ohbe_field = 'ohbe_' . $field['id'];
        $sanizited_value = isset($_POST[$ohbe_field . $extra_data])
            ? htmlspecialchars($_POST[$ohbe_field . $extra_data])
            : '';
        $required = $field['required'] ? ' required' : '';
        $middle = $extra_data == '_month' || $extra_data == '_year' ? ' middle' : '';
        $extra_data_value = $extra_data != '' ? $extra_data : '';
        $html = sprintf(
            '<select id="%1$s%5$s" name="%1$s%5$s"%2$s '
            . 'class="ohbe-select ohbe-field%3$s%4$s">',
            $ohbe_field,
            $required,
            $error_border,
            $middle,
            $extra_data_value
        );

        switch($field['id'] . $extra_data) {
            case 'country':
                $html .= OHBE_Tools::getSelectCountries($sanizited_value);
                break;
            case 'arrival_time':
                $html .= OHBE_Tools::getSelectTime('Arrival', $sanizited_value);
                break;
            case 'departure_time':
                $html .= OHBE_Tools::getSelectTime('Departure', $sanizited_value);
                break;
            case 'card_issuer';
                $html .= OHBE_Tools::getSelectCreditCardIssuers($sanizited_value);
                break;
            case 'card_expires_month':
                $html .= OHBE_Tools::getSelectMonths($sanizited_value);
                break;
            case 'card_expires_year':
                $html .= OHBE_Tools::getSelectYears($sanizited_value);
                break;
        }

        $html .= '</select>';

        return $html;
    }

    /**
     * Render input field.
     *
     * @param array $field Inventory field element.
     * @param string $type Type of input.
     * @param string $extra_data Used for differentiation (card_number|card_csc).
     * @return string Input's html structure.
     */
    private static function renderReservationFieldInput(
        $field,
        $type='text',
        $extra_data=''
    ) {
        $is_error = OHBE_Tools::hasAPIErrors($field['id']);
        $error_border = $is_error ? ' error-border' : '';
        $sanizited_value = isset($_POST['ohbe_' . $field['id']])
            ? 'value="' . htmlspecialchars($_POST['ohbe_' . $field['id']]) . '"'
            : '';
        $required = $field['required'] ? 'required' : '';
        $extra_data_value = '';
        switch($extra_data) {
            case 'card_number':
                $extra_data_value = 'data-regexp="([0-9][^0-9]*){15,}\"';
                break;
            case 'card_csc':
                $extra_data_value = 'data-regexp="([0-9]{3,4})\"';
                break;
        }

        return sprintf(
            '<input id=%1$s name=%1$s %2$s '
            . 'class="ohbe-select ohbe-field %3$s" %3$s type="%5$s" %4$s %6$s>',
            'ohbe_' . $field['id'],
            $required,
            $error_border,
            $sanizited_value,
            $type,
            $extra_data_value
        );
    }

    /**
     * Render taxes information.
     *
     * @param int $excl_taxes Quantity of taxes exclusion (%).
     * @return string Translated taxes information.
     */
    public static function renderTaxesString($excl_taxes) {
        return $excl_taxes > 0
            ? sprintf(__('Price excl. %s%% taxes', 'ohbe'), $excl_taxes)
            : __('Price incl. taxes', 'ohbe');
    }

    /**
     * Set param for saved reservation.
     *
     * @param string $key Param's key.
     * @param mixed $value Param's value.
     * @return void
     */
    public static function setReservationParam($key, $value) {
        if (!isset($_SESSION['ohbe']['reservation'])) {
            $_SESSION['ohbe']['reservation'] = array();
        }
        $_SESSION['ohbe']['reservation'][$key] = $value;
    }
}
