<?php
defined('ABSPATH') or exit();

/**
 * Redforts Oscar Hotel Booking
 * Review functions
 *
 * @author Redforts Software S.L.
 * @copyright Copyright (c) 2015 Redforts Software S.L.
 * @link https://redforts.com
 * @since 0.1.1
 */
class OHBE_Review
{
    /**
     * Print selected accommodations list.
     *
     * @param  array $reserv_data Reservation data response from Oscar.
     * @return int Number of selected items.
     */
    public static function countSelectedItems($reserv_data) {
        $count = 0;
        foreach ($reserv_data['items'] as $item) {
            if ($item['quantity'] > 0) {
                $count++;
            }
        }

        return $count;
    }

    /**
     * Get the reservation total price.
     *
     * @param  array $reserv_data Reservation data response from Oscar.
     * @return int Total price.
     */
    public static function getReservationTotalprice($reserv_data) {
        $total_price = 0;

        foreach ($reserv_data['accos'] as $acco) {
            $total_price += $acco['price'];
        }
        if (isset($reserv_data['items'])) {
            foreach ($reserv_data['items'] as $item) {
                if ($item['quantity'] > 0) {
                    $subtotal = $item['quantity'] * $item['price'];
                    if ($item['daily']) {
                        $subtotal *= $reserv_data['days'];
                    }
                    $total_price += $subtotal;
                }
            }
        }

        return OHBE_Tools::formatPrice($total_price);
    }

    /**
     * Join same accommodations in a single object.
     *
     * @param array $accos List of accommodations.
     * @return array $joined_accos Accommodation list.
     */
    public static function joinAccos($accos) {
        $inventory_accos = OHBE_Main::getInventory('accos');
        $joined_accos = array();

        foreach ($accos as $acco) {
            $id = $acco['acco_id'];
            if (array_key_exists($id, $joined_accos)) {
                $joined_accos[$id]['quantity']++;
            } else {
                $joined_accos[$id] = $acco;
                $joined_accos[$id]['quantity'] = 1;
                $joined_accos[$id]['name'] =
                    $inventory_accos[$id]['name'];
            }
        }

        return $joined_accos;
    }

    /**
     * Print items edition panel.
     *
     * @return void
     */
    public static function printEditItems() {
        $item_cards = '';
        $reserv = OHBE_Reservation::getReservationParam('data');
        $inventory_items = OHBE_Main::getInventory('items');

        if ($reserv['items'] && $inventory_items) {
            echo '<div id=items>'
                . '<h4 class="items-title">'
                . __('Have a look at our additional products and services', 'ohbe')
                . '</h4>';
            OHBE_Tools::printFlashMessage();
            echo '<div id=ohbe-description-modal class=ohbe-modal>'
                . '<div class=modal-dialog role=document>'
                . '<div class=modal-content>'
                . '<div class=modal-header>'
                . '<span class=close>&times;</span>'
                . '<h4 class=modal-title>' . __('Description', 'ohbe'). '</h4>'
                . '</div>'
                . '<div class=modal-body></div>'
                . '<div class=modal-footer>'
                . '<button type=button class="btn-ohbe btn-primary accept">'
                . __('Close', 'ohbe')
                . '</button>'
                . '</div></div></div></div>';
            if (count($reserv['items']) > 0) {
                echo '<ul class="item-list">';
                foreach ($reserv['items'] as $item) {
                    include(OHBE_PATH . 'includes/templates/views/item_card.php');
                }
                echo'</ul>';
            }

            echo '</div></div>';
        }
    }

    /**
     * Print item card's attributes.
     *
     * @param  array $item Single item structure.
     * @return void
     */
    public static function printItemCardData($item) {
        printf(
            'data-id=%s data-max="%s" data-min="%s"'
            . ' data-daily="%s" data-price=%s data-taxes="%s" data-selected="%s"',
            $item['id'],
            $item['max'],
            $item['min'],
            $item['daily'],
            $item['price'],
            $item['excl_taxes'],
            $item['quantity'] > 0
        );
    }

    /**
     * Print item quantity-total block.
     *
     * @param  array $item Item.
     * @param  int $days Number of days.
     * @return void
     */
    public static function printItemQuantityTotalBlock($item, $days) {
        $total_price = 0;
        if ($item['quantity'] > 0) {
            $total_price = $item['price'] * $item['quantity'];
            if ($item['daily']) {
                $total_price *= $days;
            }
        }
        $total_taxes = $item['excl_taxes']
            ? sprintf(
                esc_html(__('Total excluding %1$s%% taxes', 'ohbe')),
                $item['excl_taxes']
            )
            : __('Total including taxes', 'ohbe');
        printf(
            '<button type=button class="btn-quantity minus"%s>'
            . '<span class="dashicons dashicons-minus"></span></button>'
            . '<input class=value name=ohbe_quantity value=%s readonly>'
            . '<button type=button class="btn-quantity plus"%s>'
            . '<span class="dashicons dashicons-plus"></span>'
            . '</button><span class=subtotal></span>',
            $item['quantity'] <= $item['min'] ? ' disabled' : '',
            $item['quantity'],
            isset($item['max']) && $item['quantity'] >= $item['max']
                ? ' disabled'
                : ''
        );
        if ($item['daily']) {
            printf(
                '× ' .
                _n('%d day', '%d days', $days, 'ohbe'),
                $days
            );
        }
        echo '<div class=total>'
            . '<div class=ohbe-title>' . $total_taxes . ':</div>'
            . '<div class=value>' . OHBE_Tools::formatPrice($total_price) . '</div>'
            . '</div>';
    }

    /**
     * Render item price.
     *
     * @param  array $item Item.
     * @return string Price block's html structure.
     */
    public static function printPrice($item) {
        $html = sprintf(
            '<div class=price><div class=ohbe-title>%s:</div>'
            . '<div class="value">%s</div></div>',
            __('Price per item', 'ohbe'),
            OHBE_Tools::formatPrice($item['price'])
        );
        echo $html;
    }

    /**
     * Print selected accommodation cards.
     *
     * @param  array $reserv_data Reservation data response from Oscar.
     * @return void
     */
    public static function printReservationData($reserv_data) {
        $days_string = sprintf(
            _n('%s night', '%s nights', $reserv_data['days'], 'ohbe'),
            $reserv_data['days']
        );
        $rooms_string = sprintf(
            _n(
                '%s accommodation',
                '%s accommodations',
                count($reserv_data['accos']),
                'ohbe'
            ),
            count($reserv_data['accos'])
        );

        // Data
        echo '<div class="col-md-4 col-xs-12">'
            . '<h3 class=reservation-data-title>'
            . __('Reservation', 'ohbe') . ' '
            . OHBE_Reservation::getReservationParam('reserv_id')
            . '</h3>'
            . '<div class=reservation-data-panel>'
            . '<div class=divider></div><div class=data-group>';
        printf('<div class=data>%s %s %s</div>',
            isset($reserv_data['firstnames']) ? $reserv_data['firstnames'] : '',
            $reserv_data['surname'],
            isset($reserv_data['surname2']) ? $reserv_data['surname2'] : ''
        );
        echo '<div class=data>' . $reserv_data['email'] . '</div>';
        printf('<div class=data>%s</div>',
            isset($reserv_data['phone']) ? $reserv_data['phone'] : ''
        );
        // Dates
        echo '</div><div class=divider></div><div class=data-group><label>'
            . __('Arrival', 'ohbe') . ': </label><span id=arrival-date class=data>'
            . '</span><br><label>' . __('Departure', 'ohbe') . ': </label><span '
            . 'id=departure-date class=data></span></div><div class=divider></div>';
        // Accommodations
        echo '<div class=data-group>'
            . '<label>' . __('Accommodations', 'ohbe') . ': </label>'
            . '<span class=data>'
            . $days_string . ', ' . $rooms_string
            . '</span><br>';
        // Extras
        if (isset($reserv_data['items']) && count($reserv_data['items']) > 0) {
            $extras_num = 0;
            foreach ($reserv_data['items'] as $item) {
                if ($item['quantity'] > 0) {
                    $extras_num += $item['quantity'];
                }
            }
            $extras_string = sprintf(
                _n('%s extra', '%s extras', $extras_num, 'ohbe'),
                $extras_num
            );

            echo '<label>' . __('Extras', 'ohbe') . ': </label>'
                . '<span class=data>' . $extras_string . '</span>';
        }
        echo '</div><div class=divider></div>';
        // Total
        echo '<div class=data-group>'
            . '<label class=bold>Total: </label>'
            . '<span class="bold total-price">'
            . OHBE_Review::getReservationTotalprice($reserv_data)
            . '</span></div></div></div>';
    }

    /**
     * Print selected accommodations list.
     *
     * @param  array $accos Selected accommodations.
     * @param  int $itemsQuantity Amount of selected items.
     * @return void
     */
    public static function printSelectedAccos($accos, $itemsQuantity) {
        echo '<div class="col-md-8 col-xs-12 reservation-elements">'
            . '<h4>' . __('Accommodations', 'ohbe') . '</h4>'
            . '<ul class="selected-elements-list accos">';

        foreach ($accos as $acco) {
            include(
                OHBE_PATH . 'includes/templates/views/'
                . 'accommodation_reservation_data_card.php'
            );
        }

        echo '</ul>';
    }

    /**
     * Print selected accommodations list.
     *
     * @param  array $reserv_data Reservation data response from Oscar.
     * @param  int $accosQuantity Quantity of selected accommodations.
     * @param  int $itemsQuantity Quantity of selected items.
     * @return void
     */
    public static function printSelectedItems(
        $reserv_data,
        $accosQuantity,
        $itemsQuantity
    ) {
        $item_cards = '';

        if (isset($reserv_data['items']) && count($reserv_data['items']) > 0) {
            foreach ($reserv_data['items'] as $item) {
                if ($item['quantity'] > 0) {
                    ob_start();
                    include(
                        OHBE_PATH . 'includes/templates/views/'
                        . 'item_reservation_data_card.php'
                    );
                    $item_cards .= ob_get_clean();
                }
            }
        }
        if ($item_cards) {
            echo '<h4>' . __('Extras', 'ohbe') . '</h4>'
                . '<ul class="selected-elements-list extras">'
                . $item_cards
                . '</ul>';
        }

        echo '</div></div>';
    }

    /**
     * Render item card's image.
     *
     * @param  int $id Item's id.
     * @param  boolean $hasModal There is modal to render the image.
     * @return string Image's html structure.
     */
    public static function renderItemCardImage($id, $hasModal=true) {
        $html = OHBE_Tools::renderImage($id, 'items');

        if ($html && $hasModal) {
            return sprintf(
                '<a class="description modal-button" '
                . 'role=button data-modal=description-modal>%s</a>',
                $html
            );
        }

        return $html;
    }

    /**
     * Render item card's name.
     *
     * @param  array $item Item.
     * @param  boolean $hasModal There is modal to render the image.
     * @return string Name's html structure.
     */
    public static function renderItemCardName($item, $hasModal=true) {
        $inv_langs = OHBE_Main::getInventory('languages');
        $inv_lang_items = $inv_langs[OscarAPI::getLanguage()]['items'];
        $inv_lang_item = $inv_lang_items[$item['id']];
        $inv_item = OHBE_Main::getInventory('items');
        $sanitised_name = isset($inv_lang_item['name'])
            ? htmlspecialchars($inv_lang_item['name'])
            : htmlspecialchars($inv_item[$item['id']]['name']);
        $description = isset($inv_lang_item['description'])
            ? $inv_lang_item['description']
            : '';
        $image = OHBE_Tools::renderImage($item['id'], 'items');

        // Image and name
        if (($image || $description) && $hasModal) {
            return sprintf(
                '<a class="description modal-button" '
                . 'role=button data-modal=description-modal>'
                . '<div>%s</div></a>'
                . '<div class=item-description>%s</div>',
                $sanitised_name,
                $description
            );
        }

        return $sanitised_name;
    }
}
