<?php
defined('ABSPATH') or exit();

/**
 * Redforts Oscar Hotel Booking
 * Functions for different tasks
 *
 * @author Redforts Software S.L.
 * @copyright Copyright (c) 2015 Redforts Software S.L.
 * @link https://redforts.com
 * @since 0.1.1
 */
class OHBE_Tools
{
    /**
     * Check whether it is needed to update the inventory.
     *
     * @param int $version Number given by the Oscar inventory call
     * to know if inventory updated is needed.
     * @return bool
     */
    public static function checkVersion($version) {
        if (isset($version) && !OHBE_Main::getInventory('version')) {
            return true;
        }
        if (
            isset($version) && OHBE_Main::getInventory('version')
            && ($version > (int) OHBE_Main::getInventory('version'))
        ) {
            return true;
        }

        return false;
    }

    /**
     * Decode search box's filters.
     *
     * @param string $ohbe Ohbe page name parameter
     * @return mixed
     * array: All filters.
     * int: No filters.
     * object: Error(s).
     */
    public static function decodeFilters($ohbe) {
        $all_filters = array();
        if (isset($ohbe)) {
            // Arrival date
            if (isset($_GET['arrival'])) {
                $all_filters['arrival'] = DateTime::createFromFormat(
                    '!Y-m-d',
                    urldecode($_GET['arrival'])
                );
            }
            if (!isset($all_filters['arrival'])) {
                $all_filters['arrival'] = new DateTime();
            }
            // Departure date
            if (isset($_GET['departure'])) {
                $all_filters['departure'] = DateTime::createFromFormat(
                    '!Y-m-d',
                    urldecode($_GET['departure'])
                );
            }
            if (!isset($all_filters['departure'])) {
                $all_filters['departure'] = new DateTime();
                $all_filters['departure']->modify('+1 day');
            }
            // Promo code
            if (isset($_GET['pr'])) {
                $all_filters['pr'] = urldecode($_GET['pr']);
            }
        }

        return $all_filters;
    }

    /**
     * Encode search box's filters.
     *
     * @param array $url_params Params from the URL.
     * @return void
     */
    public static function encodeFilters(&$url_params) {
        // Arrival date
        if (isset($_POST['ohbe_arrival'])) {
            $arrival_date = DateTime::createFromFormat(
                '!d-m-Y',
                $_POST['ohbe_arrival']
            );
            if ($arrival_date) {
                $url_params['arrival'] = urlencode(
                    $arrival_date->format('Y-m-d')
                );
            }
        }
        // Departure date
        if (isset($_POST['ohbe_departure'])) {
            $departure_date = DateTime::createFromFormat(
                '!d-m-Y',
                $_POST['ohbe_departure']
            );
            if ($departure_date) {
                $url_params['departure'] = urlencode(
                    $departure_date->format('Y-m-d')
                );
            }
        }
        // Promo code
        if (isset($_POST['ohbe_promo'])) {
            $url_params['pr'] = urlencode($_POST['ohbe_promo']);
        }
    }

    /**
     * Get formatted price depending on inventory currency data.
     *
     * @return string Formatted price.
     */
    public static function formatPrice($price) {
        $symbol = OHBE_Main::getInventory('currency', 'symbol');
        $formatted_price = number_format(
            $price,
            OHBE_Main::getInventory('currency', 'digits')
        );

        return OHBE_Main::getInventory('currency', 'before')
            ? sprintf('%s %s', $symbol, $formatted_price)
            : sprintf('%s %s', $formatted_price, $symbol);
    }

    /**
     * Get accommodation's guest quantity.
     *
     * @param integer $acco_id Accommodation's id.
     * @param integer $rate_id Rate's id.
     * @return array Number of guests.
     */
    public static function getAccoGuests($acco_id, $rate_id) {
        $inv_rate = OHBE_Main::getInventory('rates', $rate_id);
        if (isset($inv_rate['adults'])) {
            return array(
                'adults' => $inv_rate['adults'],
                'children' => $inv_rate['children']
            );
        }
        $inv_acco = OHBE_Main::getInventory('accos', $acco_id);
        return array(
            'adults' => $inv_acco['adults'],
            'children' => $inv_acco['children']
        );
    }

    /**
     * Get accommodation name.
     *
     * @param integer $id Accommodation's id.
     * @return string The name of the accommodation.
     */
    public static function getAccoName($id) {
        $inv_lang = OHBE_Main::getInventory('languages', OscarAPI::getLanguage());
        return isset($inv_lang['accos'][$id]['name'])
            ? $inv_lang['accos'][$id]['name']
            : OHBE_Main::getInventory('accos')[$id]['name'];
    }

    /**
     * Get home URL with language slug.
     *
     * @return string URL
     */
    public static function getBaseUrl() {
        if (
            function_exists('pll_home_url')
            && pll_home_url(pll_current_language('slug'))
        ) {
            return self::getPolylangOhbePagePermalink('ohbe');
        }

        return self::getOhbePagePermalink();
    }

    /**
     * Get OHBE page permalink for Polylang.
     *
     * @param $page_slug string Page slug.
     * @return string Page permalink or home URL if the page is not found.
     */
    public static function getPolylangOhbePagePermalink($page_slug) {
        $page = get_page_by_path($page_slug);

        if(empty($page_slug) || empty($page) || is_null($page)) {
            return home_url('/');
        }

        $page_ID_current_lang = pll_get_post($page->ID);

        return empty($page_ID_current_lang)
            ? get_permalink($page->ID)
            : get_permalink($page_ID_current_lang);
    }

    /**
     * Get OHBE page permalink without languages.
     *
     * @return string URL
     */
    public static function getOhbePagePermalink() {
        return ($page = get_page_by_path('ohbe'))
            ? get_page_link($page)
            : home_url('/');
    }

    /**
     * Get nights interval between arrival and departure.
     *
     * @param array $acco Accommodation.
     * @return string Nights' quantity.
     */
    public static function getNightsInterval($acco) {
        $datetime1 = new DateTime($acco['arrival']);
        $datetime2 = new DateTime($acco['departure']);

        return $datetime1->diff($datetime2)->days;
    }

    /**
     * Get filters from search form.
     *
     * @param string $formatted Format for dates. Example: 'Y-m-d'.
     * @return array Filters.
     */
    public static function getSearchFilters($formatted=null) {
        if (isset($_GET['ohbe'])) {
            $all_filters = self::decodeFilters($_GET['ohbe']);
        }

        $arrival = isset($all_filters) ? $all_filters['arrival'] : '';
        $departure = isset($all_filters) ? $all_filters['departure'] : '';

        return array(
            'arrival' => $formatted && isset($_GET['ohbe'])
                ? $arrival->format($formatted)
                : $arrival,
            'departure' => $formatted && isset($_GET['ohbe'])
                ? $departure->format($formatted)
                : $departure,
            'promotion_code' => isset($_GET['ohbe']) && isset($all_filters['pr'])
            ? $all_filters['pr'] : ''
        );
    }

    /**
     * Get the list of countries as select's options
     *
     * @param string $country Selected country.
     * @return string Country options
     */
    public static function getSelectCountries($country=null) {
        $html = sprintf(
            '<option value="" %s disabled>%s</option>',
            $country ? '' : ' selected',
            __('Select a country', 'ohbe')
        );
        $inv_langs = OHBE_Main::getInventory('languages');
        $lang = OscarAPI::getLanguage();
        if (
            isset($inv_langs)
            && isset($inv_langs[$lang])
        ) {
            $countries = $inv_langs[$lang]['countries'];
            asort($countries);

            foreach ($countries as $key => $value) {
                $html .= sprintf(
                    '<option value=%s%s>%s</option>',
                    $key,
                    $country && $country == $key ? ' selected' : '',
                    $value
                );
            }
        }

        return $html;
    }

    /**
     * Get the list of credit card issuers as select's options
     *
     * @param string $issuer Selected month.
     * @return string Credit card issuer options
     */
    public static function getSelectCreditCardIssuers($issuer=null) {
        $html = sprintf(
            '<option value="" %s disabled>%s</option>',
            $issuer ? '' : ' selected',
            __('Select an issuer', 'ohbe')
        );
        $inv_credit_card_issuers = OHBE_Main::getInventory('creditcard_issuers');
        if (isset($inv_credit_card_issuers)) {
            foreach ($inv_credit_card_issuers as $key => $value) {
                $html .= sprintf(
                    '<option value="%1$s"%2$s>%1$s</option>',
                    $value,
                    $issuer && $issuer == $value ? ' selected' : ''
                );
            }
        }

        return $html;
    }

    /**
     * Get accommodation from selected accommodation list.
     *
     * @param array $acco Accommodation to compare with.
     * @return array Selected accommodation.
     */
    public static function getSelectedAcco($acco) {
        if (isset($_SESSION['ohbe']['selected_accos'])) {
            foreach ($_SESSION['ohbe']['selected_accos'] as $selected_acco) {
                if ($selected_acco['acco_id'] == $acco['acco_id']) {
                    return $selected_acco;
                }
            }
        }

        return null;
    }

    /**
     * Get the list of months as select's options
     *
     * @param string $month Selected month.
     * @return string Month options
     */
    public static function getSelectMonths($month=null) {
        $html = sprintf(
            '<option value="" %s disabled>%s</option>',
            $month ? '' : ' selected',
            __('Month', 'ohbe')
        );
        for ($i=1; $i<13; $i++) {
            $value = $i < 10 ? str_pad($i, 2, '0', STR_PAD_LEFT) : $i;
            $html .= sprintf(
                '<option value=%1$s%2$s>%1$s</option>',
                $value,
                $month && $month == $value ? ' selected' : ''
            );
        }

        return $html;
    }

    /**
     * Get the list of time as select's options
     *
     * @param string $type Type of select field.
     * @param string $time Selected time.
     * @return string Time options
     */
    public static function getSelectTime($type, $time=null) {
        $placeholder = $type == 'Arrival'
            ? __('Select a check-in time', 'ohbe')
            : __('Select a check-out time', 'ohbe');
        $html = sprintf(
            '<option value="" %s disabled>%s',
            $time ? '' : ' selected',
            $placeholder
        );
        for ($i=0; $i<24; $i++) {
            $html .= sprintf(
                '<option value="%02d:00"%s>%02d:00 - %02d:00',
                $i,
                $time && $time == $i ? ' selected' : '',
                $i,
                ($i + 1) % 24
            );
        }

        return $html;
    }

    /**
     * Get the list of years as select's options
     *
     * @param string $year Selected year.
     * @return string Year options
     */
    public static function getSelectYears($year=null) {
        $html = sprintf(
            '<option value="" %s disabled>%s</option>',
            $year ? '' : ' selected',
            __('Year', 'ohbe')
        );
        for ($i=0; $i<=10; $i++) {
            $value = date("Y") + $i;
            $html .= sprintf(
                '<option value=%1$s%2$s>%1$s</option>',
                date("Y") + $i,
                $year && $year == $value ? ' selected' : ''
            );
        }

        return $html;
    }

    /**
     * Check whether API has errors.
     *
     * @param string $param Search for a single API error.
     * @return boolean
     */
    public static function hasAPIErrors($param=null) {
        if (isset($_SESSION['ohbe']['api_errors'])) {
            $errors = $_SESSION['ohbe']['api_errors'];
            return isset($param) ? array_key_exists($param, $errors) : true;
        }

        return false;
    }

    /**
     * Get localized parameters for search datepicker.
     *
     * @return array Localized parameters.
     */
    public static function localizeDatepicker() {
        global $wp_locale;
        $datepicker_args = array(
            'monthNames' => self::stripArrayIndexes($wp_locale->month),
            'monthNamesShort' => self::stripArrayIndexes($wp_locale->month_abbrev),
            'dayNames' => self::stripArrayIndexes($wp_locale->weekday),
            'dayNamesShort' => self::stripArrayIndexes($wp_locale->weekday_abbrev),
            'dayNamesMin' => self::stripArrayIndexes($wp_locale->weekday_initial),
            'firstDay' => get_option('start_of_week'),
            'isRTL' => ($wp_locale->text_direction == 'rtl') ? true : false,
        );

        return $datepicker_args;
    }

    /**
     * Set an alert when there is no inventory.
     *
     * @return void
     */
    public static function mustSetInventory() {
        OHBE_Tools::setFlashMessage(
            __(
                'The inventory must be updated. Please, go to the plugin '
                    . 'configuration page and save the settings for the '
                    . 'inventory to be updated',
                'ohbe'
            ),
            'danger'
        );
        OHBE_Tools::printFlashMessage();
    }

    /**
     * Print booked list bottom menu.
     *
     * @param float $price Total price.
     * @param string $btn_id Button's id.
     * @param string $btn_name Button's name.
     * @param string $btn_text Text to be displayed in button.
     * @param bool $btn_right Button's position.
     * @param float $orig_price Original price without fixed discount.
     * @return void
     */
    public static function printBookedListBottom(
        $price,
        $btn_id,
        $btn_name,
        $btn_text,
        $btn_right = true,
        $orig_price = null
    ) {
        $btn_pos = $btn_right ? ' btn-right' : '';

        echo '<div class=booked-list-bottom>'
            . '<button type=submit id=' . $btn_id . ' name=' . $btn_name
            . ' class="btn-ohbe ' . $btn_pos . '">' . $btn_text . '</button>'
            . '<div class=total-block>'
            . '<div class=ohbe-title>' . __('Total price', 'ohbe') . ':</div>'
            . '<div class="value orig">'
            . (isset($orig_price) ? OHBE_Tools::formatPrice($orig_price) : '')
            . '</div><div class="value no-orig">'
            . OHBE_Tools::formatPrice($price)
            . '</div></div><div class=clearfix></div></div>';
    }

    /**
     * Print flash message.
     *
     * @return void
     */
    public static function printFlashMessage() {
        if (isset($_SESSION['ohbe']['flash_message'])) {
            printf(
                '<div class="alert alert-%s">%s</div>',
                $_SESSION['ohbe']['flash_message']['type'],
                $_SESSION['ohbe']['flash_message']['message']
            );
            unset($_SESSION['ohbe']['flash_message']);
        }
    }

    /**
     * Remove saved API errors.
     *
     * @return void
     */
    public static function removeApiErrors() {
        unset($_SESSION['ohbe']['api_errors']);
    }

    /**
     * Remove selected accommodations.
     *
     * @return void
     */
    public static function removeSelectedAccos() {
        unset($_SESSION['ohbe']['selected_accos']);
    }

    /**
     * Render booking summary form URL.
     *
     * @param string $page Page name
     * @param bool $must_keep_parameters Decide if the current URL params must be used.
     * @param array $drop_params_list Params to be removed.
     * @return string URL with params if exist
     */
    public static function renderFormAction(
        $page,
        $must_keep_parameters=false,
        $drop_params_list=[]
    ) {
        $url_params = OHBE_Tools::setUrlParams(
            $page,
            $must_keep_parameters,
            $drop_params_list
        );

        return esc_url_raw(add_query_arg($url_params, OHBE_Tools::getBaseUrl()));
    }

    /**
     * Render image.
     *
     * @param int $id id.
     * @param string $type Type of list to be found.
     * @return string Image's html structure.
     */
    public static function renderImage($id, $type) {
        $inventory_images = OHBE_Main::getInventory('images');
        $inventory_types = OHBE_Main::getInventory($type);

        if ($inventory_types) {
            $type_images = $inventory_types[$id]['images'];
            $image_id = isset($type_images[0]) ? $type_images[0] : null;

            if ($inventory_images && $image_id) {
                return sprintf(
                    '<div class=ohbe-image style="background-image:url(%s);"></div>',
                    $inventory_images[$image_id]['url']
                );
            }

            if (!$image_id) {
                return sprintf(
                    '<div class=ohbe-image><div class=no-image>%s</div></div>',
                    _('No image available')
                );
            }
        }
        // return empty placeholder to prevent different heights
        return '';
    }

    /**
     * Render occupancy icons for a rate.
     *
     * @param string $id Rate id
     * @param string $type Type of list to get icons from.
     * @return string Occupancy icon's html structure.
     */
    public static function renderOccupancyIcons($id, $type) {
        $list = OHBE_Main::getInventory($type);
        $occupancy = '';
        $rate = isset($list) && $list[$id] ? $list[$id] : null;

        // Show icons when rate has adults/children
        if ($rate && isset($rate['adults']) && isset($rate['children'])) {
            $adult = sprintf(
                '<div class="dashicons dashicons-admin-users" '
                . 'title="%s"></div>',
                __('Adults', 'ohbe')
            );
            $occupancy .= $rate['adults'] > 5
                ? $adult . "×" . $rate['adults']
                : str_repeat($adult, $rate['adults']);
            if (
                $rate['adults']
                && $rate['children']
                && ($rate['adults'] > 5 || $rate['children'] > 5)
            ) {
                $occupancy .= ' + ';
            }
            $children = sprintf(
                '<div class="dashicons dashicons-universal-access" '
                . 'title="%s"></div>',
                __('Children', 'ohbe')
            );
            $occupancy .= $rate['children'] > 5
                ? $children . "×" . $rate['children']
                : str_repeat($children, $rate['children']);
        }

        return $occupancy;
    }

    /**
     * Set param for API errors.
     *
     * @param string $key Param's key.
     * @param mixed $value Param's value.
     * @return void
     */
    public static function setApiErrors($key, $value) {
        if (!isset($_SESSION['ohbe']['api_errors'])) {
            $_SESSION['ohbe']['api_errors'] = array();
        }
        $_SESSION['ohbe']['api_errors'][$key] = $value;
    }

    /**
     * Set flash message.
     *
     * @param string $message Message to be displayed.
     * @param string $type Type of alert [success, danger].
     * @return void
     */
    public static function setFlashMessage($message, $type = 'success') {
        $_SESSION['ohbe']['flash_message'] = array(
            'message' => $message,
            'type' => $type
        );
    }

    /**
     * Set the URL params
     *
     * @param string $page The page's name.
     * @param bool $must_keep_parameters Decide if the current URL params must be used.
     * @param array $drop_params_list Params to be removed.
     * @return array All the params used for the new URL.
     */
    public static function setUrlParams(
        $page,
        $must_keep_parameters=false,
        $drop_params_list=[]
    ) {
        $url_params = array();
        if (!empty($_GET) && $must_keep_parameters) {
            foreach ($_GET as $key => $value) {
                if (!in_array($key, $drop_params_list)) {
                    $url_params[$key] = urlencode($value);
                }
            }
        } else if (isset($_GET['lang'])) {
            $url_params['lang'] = urlencode($_GET['lang']);
        }
        $url_params['ohbe'] = $page;

        // SEARCH FORM
        // Arrival date
        if (isset($_POST['ohbe_arrival'])) {
            $arrival_date = DateTime::createFromFormat(
                '!d-m-Y',
                $_POST['ohbe_arrival']
            );
            if ($arrival_date) {
                $url_params['arrival'] = urlencode(
                    $arrival_date->format('Y-m-d')
                );
            }
        }
        // Departure date
        if (isset($_POST['ohbe_departure'])) {
            $departure_date = DateTime::createFromFormat(
                '!d-m-Y',
                $_POST['ohbe_departure']
            );
            if ($departure_date) {
                $url_params['departure'] = urlencode(
                    $departure_date->format('Y-m-d')
                );
            }
        }
        // Promo code
        if (isset($_POST['ohbe_promo'])) {
            $url_params['pr'] = urlencode($_POST['ohbe_promo']);
        }

        return $url_params;
    }

    /**
     * Sort price by ASC.
     *
     * @return int Difference between prices.
     */
    public static function sortByPrice($a, $b) {
        return $a['price'] - $b['price'];
    }

    /**
     * Remove "string" indexes from array.
     *
     * @return array Array without "string" indexes
     */
    public static function stripArrayIndexes($array) {
        foreach ($array as $item) {
            $array_stripped[] = $item;
        }

        return $array_stripped;
    }

    /**
     * Validate search form dates.
     *
     * @param string $post_value Name of the post variable
     * @return boolean Tells whether the date is valid or not.
     */
    public static function validateDate($post_value) {
        return !isset($_POST[$post_value])
            || empty($_POST[$post_value])
            || !preg_match('/[0-9]{2}-[0-9]{2}-[0-9]{4}/', $_POST[$post_value]);
    }
}
