<?php
defined('ABSPATH') or exit();

/**
 * Redforts Oscar Hotel Booking
 * Front core functions
 *
 * @author Redforts Software S.L.
 * @copyright Copyright (c) 2015 Redforts Software S.L.
 * @link https://redforts.com
 * @since 0.1.1
 */

/**
 * Enqueue Dashicons style to frontend.
 *
 * @return void
 */
function addDashicons() {
    wp_enqueue_style('dashicons');
}

add_action('wp_enqueue_scripts', 'addDashicons', 999);

/**
 * Handles accommodations availability page.
 *
 * @return mixed
 *      array When param "ohbe" is set & filters are set
 *      int When param "ohbe" is set & no filters are set
 *      null When param "ohbe" is not set & no errors are set
 *      object When errors are set
 */
function ohbeAvailabilityFormHandle() {
    if (isset($_GET['ohbe'])) {
        $all_filters = OHBE_Tools::decodeFilters($_GET['ohbe']);
        if (array_key_exists('errors', $all_filters)) {
            return $all_filters;
        }
        $errors = OHBE_Availability::accommodationAvailabilityErrors(
            'position',
            $all_filters
        );
    }
    else {
        $errors = OHBE_Availability::accommodationAvailabilityErrors('params');
    }
    if ($errors) {
        return $errors;
    }
}

/**
 * Handle search errors.
 *
 * @return object|void Search errors.
 */
function ohbeSearchErrors() {
    $errors_ohbe = new WP_Error();

    if (OHBE_Tools::validateDate('ohbe_arrival')) {
        $errors_ohbe->add('ohbeCheckIn', __('Invalid arrival date', 'ohbe'));
        return $errors_ohbe;
    }

    if (OHBE_Tools::validateDate('ohbe_departure')) {
        $errors_ohbe->add('ohbeCheckOut', __('Invalid departure date', 'ohbe'));
        return $errors_ohbe;
    }
}

/**
 * Handle accommodations search form.
 * Redirect to availability page.
 *
 * @return object When errors are set
 */
function ohbeSearchFormHandle($id) {
    $errors = ohbeSearchErrors();
    if ($errors) {
        return $errors;
    }

    OHBE_Main::checkOhbePage();
    $url_params = OHBE_Tools::setUrlParams(OHBE_AVAILABILITY_PAGE);

    if ($id) {
        $url_params['acco'] = $id;
    }
    // Remove selected accommodations previously
    unset($_SESSION['ohbe']['selected_accos']);
    $url = esc_url_raw(add_query_arg($url_params, OHBE_Tools::getBaseUrl()));
    wp_safe_redirect($url);
    exit;
}

/**
 * Handle reservation page.
 * Redirect to availability or to review page.
 *
 * @return mixed
 *      array|object Errors from Oscar reservation call response
 *      int When wrong date is set
 *      void When no errors are set
 */
function ohbeReservationFormHandle() {
    wp_enqueue_script('ohbe_reservation');

    // Modify Booking
    if (
        $_SERVER['REQUEST_METHOD'] == "POST"
        && isset($_POST['modify_submit'])
    ) {
        // Redirect to Availability page
        wp_safe_redirect(OHBE_Tools::renderFormAction(OHBE_AVAILABILITY_PAGE, true));
        exit;
    }

    // Booking
    if (
        $_SERVER['REQUEST_METHOD'] == "POST"
        && (isset($_POST['action']) && $_POST['action'] == 'make_reservation')
    ) {
        $search_filters = OHBE_Tools::getSearchFilters('Y-m-d');
        $reservation_data = OHBE_Reservation::getReservationFieldValues();
        $accos = OHBE_Reservation::getAccommodationStructure();

        writeLog(
            __CLASS__ . ' '
            . __FUNCTION__ . ' '
            . session_id() . ' '
            . 'accos: ' . json_encode($accos) . ' '
            . 'count: ' . count($accos) . ' '
            . 'sess: ' . json_encode($_SESSION['ohbe']['selected_accos'])
        );

        if (count($accos) == 0) {
            OHBE_Tools::removeSelectedAccos();
            OHBE_Tools::setFlashMessage(
                __(
                    'There are no accommodations selected. Please, try again.',
                    'ohbe'
                ),
                'danger'
            );
            // Redirect to Availability page
            wp_safe_redirect(OHBE_Tools::renderFormAction(OHBE_AVAILABILITY_PAGE, true));
            exit;
        }

        $redforts_data = OscarAPI::setReservation(
            $_SESSION['ohbe']['selected_accos'][0]['arrival'],
            $_SESSION['ohbe']['selected_accos'][0]['departure'],
            'confirmed',
            $accos,
            $reservation_data,
            $search_filters['promotion_code']
        );

        if (isset($redforts_data['errors'])) {
            writeLog(__FUNCTION__ . ' ' . json_encode($redforts_data['errors']));
            foreach ($redforts_data['errors'] as $error) {
                $isRedirect = OHBE_Reservation::handleReservationError($error);
                if ($isRedirect) {
                    // Redirect to Availability page
                    wp_safe_redirect(
                        OHBE_Tools::renderFormAction(OHBE_AVAILABILITY_PAGE, true)
                    );
                    exit;
                }
            }

            return;
        }

        $arrival_date = DateTime::createFromFormat(
            '!Y-m-d',
            $_SESSION['ohbe']['selected_accos'][0]['arrival']
        );
        OHBE_Reservation::setReservationParam('arrival', $arrival_date);
        OHBE_Reservation::setReservationParam('reserv_id', $redforts_data['id']);
        OHBE_Reservation::setReservationParam(
            'name',
            OHBE_Reservation::getReservationFullName($reservation_data)
        );
        OHBE_Reservation::setReservationParam(
            'data',
            OscarAPI::getReservationData()
        );
        OHBE_Tools::setFlashMessage(
            __(
                'An email message has been sent to you. If you do not find '
                    . 'it in your Inbox, check the Spam folder and mark the email '
                    . '"Not spam".',
                'ohbe'
            )
        );

        unset($_SESSION['ohbe']['selected_accos']);
        // Authorize access to Reservation complete page
        $_SESSION['ohbe']['is_reservation_complete'] = true;
        // Redirect to Reservation complete page
        wp_safe_redirect(OHBE_Tools::renderFormAction(
            OHBE_RESERVATION_COMPLETE_PAGE
        ));
        exit;
    }
    if (isset($_GET['ohbe'])) {
        $all_filters = OHBE_Tools::decodeFilters($_GET['ohbe']);
        $errors = OHBE_Availability::accommodationAvailabilityErrors(
            'position',
            $all_filters
        );
        if ($errors) {
            return $errors;
        }
    }
    else {
        $errors = OHBE_Availability::accommodationAvailabilityErrors('params');
        if ($errors) {
            return $errors;
        }
    }
}

add_action('init', 'ohbeReservationFormHandle');

/**
 * Handle reservation complete page.
 * Redirect to review page when a reservation hasn't been made but reservation
 * data is saved.
 * Redirect to home when no reservation data is saved.
 *
 * @return void
 */
function ohbeReservationCompleteHandle() {
    if (
        $_SERVER['REQUEST_METHOD'] != "POST"
        && isset($_GET['ohbe'])
        && $_GET['ohbe'] == OHBE_RESERVATION_COMPLETE_PAGE
        && !isset($_SESSION['ohbe']['is_reservation_complete'])
    ) {
        if (OHBE_Reservation::getReservationParam()) {
            $url_params['ohbe'] = OHBE_REVIEW_PAGE;
            $url = esc_url_raw(add_query_arg($url_params, OHBE_Tools::getBaseUrl()));
        } else {
            $url = esc_url_raw(OHBE_Tools::getBaseUrl());
        }
        wp_safe_redirect($url);
        exit;
    }
}

add_action('init', 'ohbeReservationCompleteHandle');

/**
 * Handle review page forms.
 *
 * @return void
 */
function ohbeReviewFormHandle() {
    if (
        isset($_GET['ohbe'])
        && (
            $_GET['ohbe'] == OHBE_REVIEW_PAGE
            || (
                $_GET['ohbe'] == OHBE_RESERVATION_COMPLETE_PAGE
                && isset($_SESSION['ohbe']['is_reservation_complete'])
            )
        )
    ) {
        // Ensure no one can access to reservation complete page
        // unless a reservation has been made
        unset($_SESSION['ohbe']['is_reservation_complete']);
        // Update item
        if (
            $_SERVER['REQUEST_METHOD'] == "POST"
            && isset($_POST['item_submit'])
            && OHBE_Reservation::getReservationParam()
        ) {
            $redforts_data = OscarAPI::setItemsUpdate(array(
                'id' => intval($_POST['ohbe_id']),
                'quantity' => intval($_POST['ohbe_quantity']),
            ));
            if (isset($redforts_data['errors'])) {
                OHBE_Tools::setFlashMessage(
                    $redforts_data['errors'][0]['text'],
                    'danger'
                );
            }
            else {
                OHBE_Tools::setFlashMessage(
                    __(
                        'An email message has been sent to you. If you do not find '
                            . 'it in your Inbox, check the Spam folder and mark '
                            . 'the email "Not spam".'
                        , 'ohbe'
                    )
                );
            }
        }

        // Ensure that reservations params are defined
        if (OHBE_Reservation::getReservationParam()) {
            $reservation_data = OscarAPI::getReservationData();
            OHBE_Reservation::setReservationParam('data', $reservation_data);
            if (isset($reservation_data['errors'])) {
                OHBE_Tools::setFlashMessage(
                    $reservation_data['errors'][0]['text'],
                    'danger'
                );
                OHBE_Reservation::deleteReservationParam();
            }
        }
    }
}

add_action('init', 'ohbeReviewFormHandle');

function writeLog($arg='') {
    if (!OHBE_LOG) {
        return;
    }

    $log_file = date('Ymd') . '.log';
    $log_limit = date('Ymd', strtotime('-7 day')) . '.log';
    $log_dir = OHBE_PATH . 'logs/';

    if (!file_exists($log_dir)) {
        mkdir($log_dir);
    }

    foreach (scandir($log_dir) as $f) {
        if (preg_match('/\d{8}\.log/', $f) && $f < $log_limit) {
            unlink($log_dir . '/' . $f);
        }
    }

    $ip = array_key_exists('HTTP_X_FORWARDED_FOR', $_SERVER)
        ? $_SERVER['HTTP_X_FORWARDED_FOR']
        : $_SERVER['REMOTE_ADDR'];
    $msg = date('c') . ' ' . $ip;

    if (is_array($arg) || is_object($arg)) {
        $msg .= ' ' . json_encode($arg);
    } else {
        $msg .= ' ' . $arg;
    }

    $msg .= "\n";


    $handle = fopen($log_dir . $log_file, 'a');
    fwrite($handle, $msg);
    fclose($handle);
}

function hasCookie() {
    return isset($_COOKIE[session_name()]);
}
